'From Squeak 2.0 BETA of May 8, 1998 on 15 May 1998 at 6:59:58 am'!
"Change Set:		jhmNewBookPageProto
Date:			15 May 1998
Author:			John Maloney

Adds a facility to BookMorph that allows a page to be
recorded as a prototype to be copied when inserting new
pages. Handy for making a book with a consistent layout.
Note that this is not a master-page facility; a change to
the prototype page is not propagated to pages created
from the prototype before the change."
!

AlignmentMorph subclass: #BookMorph
	instanceVariableNames: 'pageSize pages currentPage copyContents newPagePrototype '
	classVariableNames: 'PageFlipSoundOn '
	poolDictionaries: ''
	category: 'Morphic-Widgets'!

!BookMorph methodsFor: 'menu' stamp: 'jm 5/15/1998 06:45'!
addBookMenuItemsTo: aCustomMenu hand: aHandMorph
	aCustomMenu
		add: (copyContents
				ifTrue: ['don''t be parts bin when closed']
				ifFalse: ['be parts bin when closed'])
		action: #toggleCopyContents.
	aCustomMenu add: 'previous page' action: #previousPage.
	aCustomMenu add: 'next page' action: #nextPage.
	aCustomMenu add: 'insert a page' action: #insertPage.
	aCustomMenu add: 'delete this page' action: #deletePage.
	aCustomMenu add: 'page controls' action: #pageControls:.
	aCustomMenu add: 'sort pages' action: #sortPages:.
	aCustomMenu add: 'save as new-page prototype' action: #setNewPagePrototype.
	newPagePrototype ifNotNil: [
		aCustomMenu add: 'clear new-page prototype' action: #clearNewPagePrototype].

	(aHandMorph classOfPasteBuffer isKindOf: PasteUpMorph class) ifTrue:
		[aCustomMenu add: 'paste book page'	action: #pasteBookPage]
! !

!BookMorph methodsFor: 'menu' stamp: 'jm 5/14/1998 20:18'!
clearNewPagePrototype
	"Clear the new page prototype."

	newPagePrototype _ nil.
! !

!BookMorph methodsFor: 'menu' stamp: 'jm 5/14/1998 20:37'!
insertPageColored: aColor

	| sz newPage bw bc |
	currentPage == nil
		ifTrue:
			[sz _ pageSize.
			bw _ 0.
			bc _ Color blue muchLighter]
		ifFalse:
			[sz _ currentPage extent.
			bw _ currentPage borderWidth.
			bc _ currentPage borderColor].
	newPagePrototype
		ifNil: [
			newPage _ PasteUpMorph new extent: sz; color: aColor.
			newPage borderWidth: bw; borderColor: bc]
		ifNotNil: [
			newPage _ newPagePrototype fullCopy].
	newPage resizeToFit: false.
	pages isEmpty
		ifTrue: [pages add: (currentPage _ newPage)]
		ifFalse: [pages add: newPage after: currentPage].
	self nextPage.
! !

!BookMorph methodsFor: 'menu' stamp: 'jm 5/15/1998 06:46'!
invokeBookMenu
	"Answer a menu to be popped up from the book-control panel"
	| aMenu |
	aMenu _ CustomMenu new.
	aMenu addList:	#(
		"	('border color...' 		changeBorderColor:)
			('border width...' 		changeBorderWidth:)
			('lock'					lock)"
			('make bookmark'		bookmarkForThisPage)
			('sort pages'				sortPages:)
			('remove control panel'	deleteControls)
		).
	(self primaryHand classOfPasteBuffer isKindOf: PasteUpMorph class) ifTrue:
		[aMenu add: 'paste book page'	action: #pasteBookPage].

	aMenu add: 'save as new-page prototype' action: #setNewPagePrototype.
	newPagePrototype ifNotNil: [
		aMenu add: 'clear new-page prototype' action: #clearNewPagePrototype].

	aMenu add: (openToDragNDrop ifTrue: ['close'] ifFalse: ['open']) , ' dragNdrop'
			action: #openCloseDragNDrop.

	aMenu invokeOn: self defaultSelection: nil! !

!BookMorph methodsFor: 'menu' stamp: 'jm 5/14/1998 20:17'!
setNewPagePrototype
	"Record the current page as the prototype to be copied when inserting new pages."

	currentPage ifNotNil:
		[newPagePrototype _ currentPage fullCopy].
! !

!BookMorph methodsFor: 'object fileIn' stamp: 'jm 5/15/1998 06:59'!
convertbosfcepcbbochvimolppcc0: varDict bosfcepcbbochvimolppccn0: smartRefStrm
	"Adding newPagePrototype instance variable."
	
	newPagePrototype _ nil.
! !


DriveACar removeSelector: #title!
FindTheLight removeSelector: #title!
Smalltalk removeClassNamed: #ColorMemoryMorph!
