'From Squeak 2.4c of May 10, 1999 on 18 July 1999 at 9:27:07 pm'!
"Change Set:		definition-sw
Date:			18 July 1999
Author:			Scott Wallace

Adds a UI in the help... branch of the screen menus allowing you to look up the definition for any English word, using the WordNet lexicon at Princeton Univ."!


!HandMorph methodsFor: 'world menu' stamp: 'sw 7/18/1999 20:51'!
helpMenu
	"Build the help menu for the world."
	| menu screenCtrl |
	screenCtrl _ ScreenController new.
	menu _ (MenuMorph entitled: 'help...') defaultTarget: self.
	menu addStayUpItem.
	menu add: 'about this system...' target: Smalltalk action: #aboutThisSystem.
	menu balloonTextForLastItem: 'current version information.'.
	menu add: 'update code from server' action: #absorbUpdatesFromServer.
	menu balloonTextForLastItem: 'load latest code updates via the internet'.
	menu add: 'preferences...' target: Preferences action: #openPreferencesInspector.
	menu balloonTextForLastItem: 'view and change various options.'.
	menu addLine.

	menu add: 'command-key help' target: Utilities action: #openCommandKeyHelp.
	menu balloonTextForLastItem: 'summary of keyboard shortcuts.'.
	menu add: 'world menu help' target: self action: #worldMenuHelp.
	menu balloonTextForLastItem: 'helps find menu items buried in submenus.'.
	"menu add: 'info about flaps' target: Utilities action: #explainFlaps."
	menu balloonTextForLastItem: 'describes how to enable and use flaps.'.
	menu add: 'font size summary' target: Utilities action: #fontSizeSummary.
	menu balloonTextForLastItem: 'summary of names and sizes of available fonts.'.
	menu add: 'useful expressions' target: Utilities action: #openStandardWorkspace.
	menu balloonTextForLastItem: 'a window full of useful expressions.'.
	menu add: 'view graphical imports' target: Smalltalk action: #viewImageImports.
	menu balloonTextForLastItem: 'lets you look at graphical imports.'.
	menu addLine.

	menu addUpdating: #gridOnString action: #setGridding.
	menu balloonTextForLastItem: 'turn gridding on or off.'.
	menu add: 'telemorphic...' action: #remoteDo.
	menu balloonTextForLastItem: 'commands for doing multi-machine "telemorphic" experiments'.
	menu addUpdating: #soundEnablingString target: Preferences action: #toggleSoundEnabling.
	menu balloonTextForLastItem: 'turning sound off will completely disable Squeak''s use of sound.'.
	menu add: 'definition for...' target: Utilities action: #lookUpDefinition.
	menu balloonTextForLastItem: 'if connected to the internet, use this to look up the definition of an English word.'.
	menu addLine.

	menu add: 'set author initials...' target: screenCtrl action: #setAuthorInitials.
	menu balloonTextForLastItem: 'supply initials to be used to identify the author of code and other content.'.
	menu add: 'vm statistics' target: screenCtrl action: #vmStatistics.
	menu balloonTextForLastItem: 'obtain some intriguing data about the vm.'.
	menu add: 'space left' target: screenCtrl action: #garbageCollect.
	menu balloonTextForLastItem: 'perform a full garbage-collection and report how many bytes of space remain in the image.'.
	^ menu
! !


!ScreenController methodsFor: 'menu messages' stamp: 'sw 7/18/1999 20:54'!
lookUpDefinition
	Utilities lookUpDefinition! !

!ScreenController methodsFor: 'nested menus' stamp: 'sw 7/18/1999 21:25'!
helpMenu 
	"Answer the help menu to be put up as a screen submenu"

	^ SelectionMenu labelList:
		#(	'keep this menu up'

			'about this system'
			'update code from server'
			'edit preferences...'

			'command-key help'
			'font size summary'
			'useful expressions'
			'view graphical imports'),

			(Array with: ((Preferences valueOfFlag: #disableSounds)
							ifTrue: ['turn sound on']
							ifFalse: ['turn sound off'])) ,

		#(	'definition for...'
			'set author initials...'
			'vm statistics'
			'space left')
		lines: #(1 4 6 10)
		selections: #(durableHelpMenu aboutThisSystem absorbUpdatesFromServer
editPreferences  openCommandKeyHelp fontSizeSummary openStandardWorkspace viewImageImports
soundOnOrOff  lookUpDefinition setAuthorInitials vmStatistics garbageCollect)
"
ScreenController new helpMenu startUp
"! !


!Utilities class methodsFor: 'miscellaneous' stamp: 'sw 7/18/1999 21:22'!
lookUpDefinition
	| aWord aDefinition |
	(aWord _ FillInTheBlank request: 'Enter a word:') isEmpty ifTrue: [^ self].
	(aDefinition _ WordNet definitionsFor: aWord) ifNil: [^ self].
	(StringHolder new contents: aDefinition)
		openLabel: aWord

"Utilities lookUpDefinition"! !


!WordNet methodsFor: 'as yet unclassified' stamp: 'sw 7/18/1999 21:12'!
definition: theWord
	"look this word up in the basic way.  Return nil if there is trouble accessing the web site."
	| doc |
	word _ theWord.
	doc _ HTTPSocket 
		httpGetDocument: 'http://www.cogsci.princeton.edu/cgi-bin/webwn/' 
		args: 'stage=1&word=', word.
	replyHTML _ (doc isKindOf: MIMEDocument)
		ifTrue:
			[doc content]
		ifFalse:
			[nil].
	"self parseReply."

	^ replyHTML! !


!WordNet class methodsFor: 'as yet unclassified' stamp: 'sw 7/18/1999 21:22'!
definitionsFor: aWord
	| aDef parts defStream item |
	aDef _ self new.
	(aDef definition: aWord) ifNil:
		[self inform: 'Sorry, cannot reach the WordNet
web site; task abandoned.'.
		^ nil].
	parts _ aDef parts.
	parts size = 0 ifTrue:
		[self inform: 'Sorry, ', aWord, ' not found.'.
		^ nil].

	defStream _ ReadWriteStream on: ''.
	defStream nextPutAll: aWord; cr.
	parts do:
		[:aPart |
			defStream cr.
			1 to: (aDef sensesFor: aPart) do:
				[:senseNumber |
					defStream nextPutAll: aPart.
					item _ aDef def: senseNumber for: aPart.
					defStream nextPutAll: (' (', senseNumber printString, ') ', (item copyFrom: 2 to: item size - 1)).
					defStream cr]].

	^ defStream contents

"WordNet definitionsFor: 'balloon'"
! !


