'From Squeak 2.1 of June 30, 1998 on 17 August 1998 at 10:31:27 am'!
"Change Set:		MoreFixes8-17-di
Date:			17 August 1998
Author:			Dan Ingalls

A simple fix to pass a model along in morphic text so that, eg, text links now work properly.

A minor tweak to make scroll speed increase gradually.

A long-overdue conversion of syntax errors so they appear in morphic, not out in MVC.
"!

Slider subclass: #ScrollBar
	instanceVariableNames: 'menuButton upButton downButton pagingArea scrollDelta pageDelta interval menuSelector timeOfMouseDown timeOfLastScroll nextPageDirection currentScrollDelay '
	classVariableNames: 'UpArrow '
	poolDictionaries: ''
	category: 'Morphic-Windows'!

!ScrollBar methodsFor: 'scrolling' stamp: 'di 8/17/1998 09:39'!
nextPage: event
	"Note: this method is no longer called, except by old scrollBars"
	(self waitForDelay1: 300 delay2: 100) ifFalse: [^ self].
	event cursorPoint y >= slider center y
		ifTrue: [self setValue: (value + pageDelta min: 1.0)]
		ifFalse: [self setValue: (value - pageDelta max: 0.0)]
! !

!ScrollBar methodsFor: 'scrolling' stamp: 'di 8/17/1998 09:39'!
scrollByPage: event
	nextPageDirection == nil ifTrue:
		[nextPageDirection _ event cursorPoint y >= slider center y].
	(self waitForDelay1: 300 delay2: 100) ifFalse: [^ self].
	nextPageDirection
		ifTrue: [self setValue: (value + pageDelta min: 1.0)]
		ifFalse: [self setValue: (value - pageDelta max: 0.0)]
! !

!ScrollBar methodsFor: 'scrolling' stamp: 'di 8/17/1998 09:40'!
scrollDown
	(self waitForDelay1: 300 delay2: 50) ifFalse: [^ self].
	self setValue: (value + scrollDelta + 0.000001 min: 1.0)! !

!ScrollBar methodsFor: 'scrolling' stamp: 'di 8/17/1998 09:40'!
scrollUp
	(self waitForDelay1: 300 delay2: 50) ifFalse: [^ self].
	self setValue: (value - scrollDelta - 0.000001 max: 0.0)! !

!ScrollBar methodsFor: 'scroll timing' stamp: 'di 8/17/1998 09:22'!
resetTimer
	timeOfMouseDown _ Time millisecondClockValue.
	timeOfLastScroll _ timeOfMouseDown - 1000 max: 0.
	nextPageDirection _ nil.
	currentScrollDelay _ nil! !

!ScrollBar methodsFor: 'scroll timing' stamp: 'di 8/17/1998 09:38'!
waitForDelay1: delay1 delay2: delay2
	"Return true if an appropriate delay has passed since the last scroll operation.
	The delay decreases exponentially from delay1 to delay2."
	| now scrollDelay |
	timeOfLastScroll == nil ifTrue: [self resetTimer]. "Only needed for old instances"
	now _ Time millisecondClockValue.
	now < timeOfLastScroll ifTrue: [self resetTimer  "rare clock rollover"].
	(scrollDelay _ currentScrollDelay) == nil ifTrue: [scrollDelay _ delay1  "initial delay"].
	now > (timeOfLastScroll + scrollDelay) ifFalse: [^ false  "not time yet"].
	currentScrollDelay _ scrollDelay*9//10 max: delay2.  "decrease the delay"
	timeOfLastScroll _ now.
	^ true! !


!SyntaxError methodsFor: 'other' stamp: 'di 8/17/1998 10:28'!
contents: aString notifying: aController
	"Compile the code in aString and notify aController of any errors. If there are no errors, then automatically proceed."

	(class compile: aString classified: category notifying: aController)
		ifNil: [^ false]
		ifNotNil: [aController hasUnacceptedEdits: false.  self proceed].
! !


!SyntaxError class methodsFor: 'instance creation' stamp: 'di 8/17/1998 10:22'!
buildMorphicViewOn: aSyntaxError
	"Answer an Morphic view on the given SyntaxError."
	| window |
	window _ (SystemWindow labelled: 'Syntax Error') model: aSyntaxError.

	window addMorph: (PluggableListMorph on: aSyntaxError list: #list
			selected: #listIndex changeSelected: nil menu: #listMenu:)
		frame: (0@0 corner: 1@0.15).

	window addMorph: (PluggableTextMorph on: aSyntaxError text: #contents
			accept: #contents:notifying: readSelection: #contentsSelection
			menu: #codePaneMenu:shifted:)
		frame: (0@0.15 corner: 1@1).

	^ window openInWorldExtent: 380@220! !

!SyntaxError class methodsFor: 'instance creation' stamp: 'di 8/17/1998 10:10'!
open: aSyntaxError
	"Answer a standard system view whose model is an instance of me."
	| topView |
	<primitive: 19> "Simulation guard"
	World ifNotNil:
		[self buildMorphicViewOn: aSyntaxError.
		Project current spawnNewProcess.
		^ Processor activeProcess suspend].
	topView _ self buildMVCViewOn: aSyntaxError.
	topView controller openNoTerminateDisplayAt: Display extent // 2.
	Cursor normal show.
	Processor activeProcess suspend.
! !


!TextMorphForEditView methodsFor: 'all' stamp: 'di 8/16/1998 21:17'!
handleInteraction: interActionBlock fromEvent: evt
	"Overridden to pass along a model to the editor for, eg, link resolution, doits, etc"
	self installEditor.  "Make sure editor is installed"
	self editor sensor: (KeyboardBuffer new startingEvent: evt).  "Fool MVC"
	self editor model: editView model.  "For evaluateSelection"
	self selectionChanged.  "Note old selection"

		interActionBlock value.

	self selectionChanged.  "Note new selection"! !


ScrollBar removeSelector: #readyForNextScroll:!
