'From Squeak2.9alpha of 12 June 2000 [latest update: #2687] on 24 September 2000 at 9:51:23 am'!
"Change Set:		PolygonRotFix
Date:			23 September 2000
Author:			Daniel Ingalls

Changes polygon rotation to conform to the new protocol for referencePosition and forwardDirection.

Because polygon bounds change with rotation, polygons must store their referencePosisiton instead of a bounds-relative rotationCenter.

Because polygons are never flexed, heading = forwardDirection = rotationDegrees for them.
"!


!PolygonMorph methodsFor: 'geometry' stamp: 'di 9/24/2000 08:44'!
extent: newExtent 
	"Not really advisable, but we can preserve most of the geometry if we don't
	shrink things too small."
	| safeExtent center |
	center _ self referencePosition.
	safeExtent _ newExtent max: 20@20.
	self setVertices: (vertices collect:
		[:p | p - center * (safeExtent asFloatPoint / (bounds extent max: 1@1)) + center])! !

!PolygonMorph methodsFor: 'private' stamp: 'di 9/24/2000 08:38'!
privateMoveBy: delta
	super privateMoveBy: delta.
	vertices _ vertices collect: [:p | p + delta].
	self arrowForms do: [:f | f offset: f offset + delta].
	curveState _ nil.  "Force recomputation"
	(self valueOfProperty: #referencePosition) doIfNotNil:
		[:oldPos | self setProperty: #referencePosition toValue: oldPos + delta]! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/23/2000 16:53'!
addFlexShell

	^ self
! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/24/2000 09:35'!
heading
	"Return the receiver's heading (in eToy terms)"
	^ self forwardDirection! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/24/2000 09:36'!
heading: newHeading
	"Set the receiver's heading (in eToy terms).
	Note that polygons never use flex shells."
	self rotationDegrees: newHeading.! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/24/2000 08:38'!
referencePosition 
	"Return the current reference position of the receiver"
	^ self valueOfProperty: #referencePosition ifAbsent: [super referencePosition]
! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/24/2000 09:21'!
rotationCenter
	"Return the rotation center of the receiver. The rotation center defines the relative offset inside the receiver's bounds for locating the reference position."
	| refPos |
	refPos _ self valueOfProperty: #referencePosition
		ifAbsent: [^ 0.5@0.5].
	^ (refPos - self bounds origin) / self bounds extent asFloatPoint! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/24/2000 09:31'!
rotationCenter: aPointOrNil
	"Set the new rotation center of the receiver. The rotation center defines the relative offset inside the receiver's bounds for locating the reference position."
	| box |
	aPointOrNil isNil
		ifTrue: [self removeProperty: #referencePosition]
		ifFalse: [box _ self bounds.
				self setProperty: #referencePosition
					toValue: box origin + (aPointOrNil * box extent)]
! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/24/2000 08:42'!
rotationDegrees

	^ self forwardDirection! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/24/2000 09:42'!
rotationDegrees: degrees 
	| flex center |
	(center _ self valueOfProperty: #referencePosition) ifNil:
		[self setProperty: #referencePosition toValue: (center _ self bounds center)].
	flex _ (MorphicTransform offset: center negated)
			withAngle: (degrees - self forwardDirection) degreesToRadians.
	self setVertices: (vertices collect: [:v | (flex transform: v) - flex offset]).
	self forwardDirection: degrees.

! !

!PolygonMorph methodsFor: 'halo control' stamp: 'di 9/24/2000 09:25'!
setRotationCenterFrom: aPoint
	"Polygons store their referencePosition."
	self setProperty: #referencePosition toValue: aPoint! !

PolygonMorph removeSelector: #assuredFlex!
