'From Squeak2.9alpha of 17 July 2000 [latest update: #2803] on 4 October 2000 at 8:35:04 am'!
"Change Set:		moreFences
Date:			4 October 2000
Author:			Bob Arning

- add a per-playfield option for fence enabling. The global preference remains as a default for playfields without an explicit definition."!


!Morph methodsFor: 'e-toy support' stamp: 'RAA 10/4/2000 08:29'!
fenceEnabled

	"in case a non-pasteUp is used as a container"

	^Preferences fenceEnabled! !


!PasteUpMorph methodsFor: 'menu & halo' stamp: 'RAA 10/4/2000 08:23'!
fenceEnabledString
	"Answer the string to be shown in a menu to represent the fence enabled status"

	^self fenceEnabled
		ifTrue:
			['<on>fence enabled']
		ifFalse:
			['<off>fence enabled']
! !

!PasteUpMorph methodsFor: 'menu & halo' stamp: 'RAA 10/4/2000 08:27'!
playfieldOptionsMenu
	"Answer an auxiliary menu with options specific to playfields -- too many to be housed in the main menu"

	| aMenu isWorld |
	isWorld _ self isWorldMorph.
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu addStayUpItem.
	aMenu add: 'save on file...' action: #saveOnFile.
	aMenu add: 'save as SqueakPage at url...' action: #saveOnURL.
	aMenu add: 'update all from resources' action: #updateAllFromResources.
	(self valueOfProperty: #classAndMethod) ifNotNil:
		[aMenu add: 'broadcast as documentation' action: #saveDocPane].
	aMenu add: 'round up strays' action: #roundUpStrays.
	aMenu balloonTextForLastItem:  'Bring back all objects whose current coordinates keep them from being visible, so that at least a portion of each of my interior objects can be seen.'.
	aMenu addLine.
	aMenu add: 'shuffle contents' action: #shuffleSubmorphs.
	aMenu balloonTextForLastItem: 'Rearranges my contents in random order'.
	self griddingOn
		ifTrue: [aMenu add: 'turn gridding off' action: #griddingOnOff.
				aMenu add: (self gridVisible ifTrue: ['hide'] ifFalse: ['show']) , ' grid'
						action: #gridVisibleOnOff.
				aMenu add: 'set grid spacing...' action: #setGridSpec]
		ifFalse: [aMenu add: 'turn gridding on' action: #griddingOnOff].
	aMenu addLine.

	#(	(autoLineLayoutString	toggleAutoLineLayout
			'whether submorphs should automatically be laid out in lines')
		(indicateCursorString	toggleIndicateCursor
			'whether the "current" submorph should be indicated with a dark black border')
		(isPartsBinString		toggleIsPartsBin
			'whether dragging an object from the interior should produce a COPY of the object')
		(isOpenForDragNDropString	toggleDragNDrop
			'whether objects can be dropped into and dragged out of me')
		(mouseOverHalosString	toggleMouseOverHalos
			'whether objects should put up halos when the mouse is over them')
		(autoExpansionString	toggleAutomaticPhraseExpansion
			'whether tile phrases, dropped on me, should automatically sprout Scriptors around them')
		(originAtCenterString	toggleOriginAtCenter
			'whether the cartesian origin of the playfield should be at its lower-left corner or at the center of the playfield')
		(showThumbnailString	toggleAlwaysShowThumbnail
			'whether large objects should be represented by thumbnail miniatures of themselves')
		(fenceEnabledString	toggleFenceEnabled
			'whether whether moving objects should stop at the edge of their container')

	) do:

			[:triplet |
				(isWorld and: [#(toggleAutoLineLayout toggleIndicateCursor toggleIsPartsBin toggleAlwaysShowThumbnail) includes: triplet second]) ifFalse:
					[aMenu addUpdating: triplet first action: triplet second.
					aMenu balloonTextForLastItem: triplet third]]. 

	aMenu addUpdating: #autoViewingString action: #toggleAutomaticViewing.
	aMenu balloonTextForLastItem:  'governs whether, when an object is touched inside me, a viewer should automatically be launched for it.'.

	((isWorld not or: [self backgroundSketch notNil]) or: [presenter isNil])
		ifTrue:
			[aMenu addLine].

	isWorld ifFalse:
		[aMenu add: 'set thumbnail height...' action: #setThumbnailHeight.
		aMenu balloonTextForLastItem: 'if currently showing thumbnails governs the standard height for them'.
		aMenu add: 'behave like a Holder' action: #becomeLikeAHolder.
		aMenu balloonTextForLastItem: 'Set properties to make this object nicely set up to hold frames of a scripted animation.'].

	self backgroundSketch ifNotNil:
		[aMenu add: 'delete background painting' action: #deleteBackgroundPainting.
		aMenu balloonTextForLastItem: 'delete the graphic that forms the background for this me.'].
	presenter ifNil:
		[aMenu add: 'make detachable' action: #makeDetachable.
		aMenu balloonTextForLastItem: 'Allow this area to be separately governed by its own controls.'].

	aMenu addLine.
	aMenu add: 'use standard texture' action: #setStandardTexture.
	aMenu balloonTextForLastItem: 'use a pale yellow-and-blue background texture here.'.
	aMenu add: 'make graph paper...' action: #makeGraphPaper.
	aMenu balloonTextForLastItem: 'Design your own graph paper and use it as the background texture here.'.
	aMenu addTitle: 'playfield options...'.

	^ aMenu
! !

!PasteUpMorph methodsFor: 'options' stamp: 'RAA 10/4/2000 08:24'!
fenceEnabled

	^ self valueOfProperty: #fenceEnabled ifAbsent: [Preferences fenceEnabled]! !

!PasteUpMorph methodsFor: 'options' stamp: 'RAA 10/4/2000 08:24'!
fenceEnabled: aBoolean

	self setProperty: #fenceEnabled toValue: aBoolean! !

!PasteUpMorph methodsFor: 'options' stamp: 'RAA 10/4/2000 08:26'!
toggleFenceEnabled
	
	self fenceEnabled: self fenceEnabled not! !


!Player methodsFor: 'scripts-standard' stamp: 'RAA 10/4/2000 08:30'!
forward: dist 

	| rho radians delta didStray p aCostume |
	(aCostume _ self costume) isInWorld ifFalse: [^ self].
	aCostume isWorldOrHandMorph ifTrue: [^ self].

	rho _ (aCostume asNumber: dist) asFloat.
	radians _ (self getHeadingUnrounded asFloat - 90.0) degreesToRadians.
	delta _ (radians cos @ radians sin) * rho.

	((aCostume owner isHandMorph not) and:
	 [aCostume owner fenceEnabled]) ifTrue:
		[(aCostume owner bounds containsRect: aCostume bounds) ifFalse:
			["If I stray out of the bounds of my owner, pull me back, but
			 without changing my heading as bounce would. Do nothing if
			 bounce has already corrected the direction."
			didStray _ false.
			((aCostume left < aCostume owner left and: [delta x < 0]) or:
			 [aCostume right > aCostume owner right and: [delta x > 0]]) ifTrue: [
				delta _ delta x negated @ delta y.
				didStray _ true].
			((aCostume top < aCostume owner top and: [delta y < 0]) or:
			 [aCostume bottom > aCostume owner bottom and: [delta y > 0]]) ifTrue: [
				delta _ delta x @ delta y negated.
				didStray _ true].
			didStray ifTrue: [aCostume makeFenceSound]]].

	"use and record the fractional position"
	p _ aCostume referencePosition + delta.
	aCostume referencePosition: p.
! !

