'From Squeak2.9alpha of 16 June 2000 [latest update: #3139] on 7 December 2000 at 12:34:13 pm'!
"Change Set:		ProjStore5-tk
Date:			7 December 2000
Author:			Ted Kaehler

	Writing a project does not try to make the Symbols be strongly held.  
	ImageSegment can be told that the roots do not have repeated objects, saving the time to test this.  Good when objects were traced by a ReferenceStream and there are 1000s of roots.
	Part of a fix to handle named players (References) correctly when read in.
	Fixes bug in SequenceableCollection>>hasEqualElements:.  Seen in RunArray>>=. "!


!ImageSegment methodsFor: 'read/write segment' stamp: 'tk 12/5/2000 11:09'!
copyFromRoots: aRootArray sizeHint: segSizeHint
	"Copy a tree of objects into a WordArray segment.  The copied objects in the segment are not in the normal Squeak space.  If this method yields a very small segment, it is because objects just below the roots are pointed at from the outside.  (See findRogueRootsImSeg: for a *destructive* diagnostic of who is pointing in.)
	Caller must hold onto Symbols.
	To go faster, make sure objects are not repeated in aRootArray and other method directly, with true."

	self copyFromRoots: aRootArray sizeHint: segSizeHint areUnique: false
! !

!ImageSegment methodsFor: 'read/write segment' stamp: 'tk 12/5/2000 10:25'!
copyFromRoots: aRootArray sizeHint: segSizeHint areUnique: areUnique
	"Copy a tree of objects into a WordArray segment.  The copied objects in the segment are not in the normal Squeak space.  
	[1] For exporting a project.  Objects were enumerated by ReferenceStream and aRootArray has them all.
	[2] For exporting some classes.  See copyFromRootsForExport:. (Caller must hold Symbols, or they will not get registered in the target system.)
	[3] For 'local segments'.  outPointers are kept in the image.
	If this method yields a very small segment, it is because objects just below the roots are pointed at from the outside.  (See findRogueRootsImSeg: for a *destructive* diagnostic of who is pointing in.)"
	| segmentWordArray outPointerArray segSize rootSet uniqueRoots |
	aRootArray ifNil: [self errorWrongState].
	uniqueRoots _ areUnique 
		ifTrue: [aRootArray]
		ifFalse: [rootSet _ IdentitySet new: aRootArray size * 3.
			uniqueRoots _ OrderedCollection new.
			1 to: aRootArray size do: [:ii |	"Don't include any roots twice"
				(rootSet includes: (aRootArray at: ii)) 
					ifFalse: [
						uniqueRoots addLast: (aRootArray at: ii).
						rootSet add: (aRootArray at: ii)]
					ifTrue: [userRootCnt ifNotNil: ["adjust the count"
								ii <= userRootCnt ifTrue: [userRootCnt _ userRootCnt - 1]]]].
			uniqueRoots].
	arrayOfRoots _ uniqueRoots asArray.
	rootSet _ uniqueRoots _ nil.	"be clean"
	userRootCnt ifNil: [userRootCnt _ arrayOfRoots size].
	arrayOfRoots do: [:aRoot | (aRoot respondsTo: #indexIfCompact) ifTrue: [
		aRoot indexIfCompact > 0 ifTrue: [
			self error: 'Compact class ', aRoot name, ' cannot be a root']].
		aRoot _ nil].	"clean up"
	outPointers _ nil.	"may have used this instance before"
	segSize _ segSizeHint > 0 ifTrue: [segSizeHint *3 //2] ifFalse: [50000].

	["Guess a reasonable segment size"
	segmentWordArray _ WordArrayForSegment new: segSize.
	[outPointerArray _ Array new: segSize // 20] ifError: [
		state _ #tooBig.  ^ self].
	"Smalltalk garbageCollect."
	(self storeSegmentFor: arrayOfRoots
					into: segmentWordArray
					outPointers: outPointerArray) == nil]
		whileTrue:
			["Double the segment size and try again"
			segmentWordArray _ outPointerArray _ nil.
			segSize _ segSize * 2].
	segment _ segmentWordArray.
	outPointers _ outPointerArray.
	state _ #activeCopy.
	endMarker _ segment nextObject. 	"for enumeration of objects"
	endMarker == 0 ifTrue: [endMarker _ 'End' clone].
! !

!ImageSegment methodsFor: 'read/write segment' stamp: 'tk 12/5/2000 09:40'!
copyFromRootsForExport: rootArray 
	"When possible, use copySmartRootsExport:.  This way may not copy a complete tree of objects.  Add to roots: all of the methods pointed to from the outside by blocks."
	| newRoots list segSize symbolHolder |
	arrayOfRoots _ rootArray.
	Smalltalk forgetDoIts.  
	symbolHolder _ Symbol allInstances.	"Hold onto Symbols with strong pointers, 
		so they will be in outPointers"
	(newRoots _ self rootsIncludingPlayers) ifNotNil: [
		arrayOfRoots _ newRoots].		"world, presenter, and all Player classes"
	"Creation of the segment happens here"
	self copyFromRoots: arrayOfRoots sizeHint: 0.
	segSize _ segment size.
	[(newRoots _ self rootsIncludingBlockMethods) == nil] whileFalse: [
		arrayOfRoots _ newRoots.
		self copyFromRoots: arrayOfRoots sizeHint: segSize].
		"with methods pointed at from outside"
	[(newRoots _ self rootsIncludingBlocks) == nil] whileFalse: [
		arrayOfRoots _ newRoots.
		self copyFromRoots: arrayOfRoots sizeHint: segSize].
		"with methods, blocks from outPointers"
	"classes of receivers of blocks"
	list _ self compactClassesArray.
	outPointers _ outPointers, ((list select: [:cls | cls ~~ nil]), (Array with: 1717 with: list)).
	"Zap sender of a homeContext. Can't send live stacks out."
	1 to: outPointers size do: [:ii | 
		(outPointers at: ii) class == BlockContext ifTrue: [outPointers at: ii put: nil].
		(outPointers at: ii) class == MethodContext ifTrue: [outPointers at: ii put: nil]].
	symbolHolder.! !

!ImageSegment methodsFor: 'read/write segment' stamp: 'tk 12/6/2000 12:40'!
copySmartRootsExport: rootArray 
	"Use SmartRefStream to find the object.  Make them all roots.  Create the segment in memory.  Project should be in first five objects in rootArray."
	| newRoots list segSize symbolHolder dummy replacements world |
	Smalltalk forgetDoIts.  
	1 to: 5 do: [:ii | ((rootArray at: ii) respondsTo: #isCurrentProject) ifTrue: [
					world _ rootArray at: ii]].
	world ifNotNil: [world presenter flushPlayerListCache].	"old and outside guys"
	symbolHolder _ Symbol allInstances.	"Hold onto Symbols with strong pointers, 
		so they will be in outPointers"

	dummy _ ReferenceStream on: (DummyStream on: nil).
		"Write to a fake Stream, not a file"
	"Collect all objects"
	dummy insideASegment: true.	"So Uniclasses will be traced"
	dummy rootObject: rootArray.	"inform him about the root"
	dummy nextPut: rootArray.
	SmartRefStream new uniClassInstVarsRefs: dummy.
		"catalog the extra objects in UniClass inst vars.  Put into dummy"
	"refs _ dummy references."
	arrayOfRoots _ self smartFillRoots: dummy.	"guaranteed none repeat"
	self savePlayerReferences: dummy references.	"for shared References table"
	replacements _ dummy blockers.
	dummy project ifNil: [self error: 'lost the project!!'].
	dummy project class == DiskProxy ifTrue: [self error: 'saving the wrong project'].
	dummy _ nil.	"force GC?"
	
	"Creation of the segment happens here"

	"try using one-quarter of memory to publish (will get bumped later)"
	self copyFromRoots: arrayOfRoots sizeHint: (Smalltalk garbageCollect // 4 // 4) areUnique: true.
	segSize _ segment size.
	[(newRoots _ self rootsIncludingBlockMethods) == nil] whileFalse: [
		arrayOfRoots _ newRoots.
		self copyFromRoots: arrayOfRoots sizeHint: segSize areUnique: true].
		"with methods pointed at from outside"
	[(newRoots _ self rootsIncludingBlocks) == nil] whileFalse: [
		arrayOfRoots _ newRoots.
		self copyFromRoots: arrayOfRoots sizeHint: segSize areUnique: true].
		"with methods, blocks from outPointers"
	list _ self compactClassesArray.
	outPointers _ outPointers, ((list select: [:cls | cls ~~ nil]), (Array with: 1717 with: list)).
	1 to: outPointers size do: [:ii | 
		(outPointers at: ii) class == BlockContext ifTrue: [outPointers at: ii put: nil].
		(outPointers at: ii) class == MethodContext ifTrue: [outPointers at: ii put: nil].
		"substitute new object in outPointers"
		(replacements includesKey: (outPointers at: ii)) ifTrue: [
			outPointers at: ii put: (replacements at: (outPointers at: ii))]].
	symbolHolder.! !

!ImageSegment methodsFor: 'read/write segment' stamp: 'tk 12/6/2000 12:45'!
savePlayerReferences: dictOfAllObjects
	| save world |
	"Save our associations in the shared References table.  They will be installed in the new system."

	save _ OrderedCollection new.
	References associationsDo: [:ass |
		(dictOfAllObjects includesKey: ass) ifTrue: [save add: ass]].
	1 to: 5 do: [:ii | ((arrayOfRoots at: ii) respondsTo: #isCurrentProject) ifTrue: [
					world _ arrayOfRoots at: ii]].
	world setProperty: #References toValue: save.
		"assume it is not refed from outside and will be traced"! !

!ImageSegment methodsFor: 'fileIn/Out' stamp: 'tk 12/6/2000 13:00'!
comeFullyUpOnReload: smartRefStream
	"fix up the objects in the segment that changed size.  An object in the segment is the wrong size for the modern version of the class.  Construct a fake class that is the old size.  Replace the modern class with the old one in outPointers.  Load the segment.  Traverse the instances, making new instances by copying fields, and running conversion messages.  Keep the new instances.  Bulk forward become the old to the new.  Let go of the fake objects and classes.
	After the install (below), arrayOfRoots is filled in.  Globalize new classes.  Caller may want to do some special install on certain objects in arrayOfRoots. 
	May want to write the segment out to disk in its new form."

	| mapFakeClassesToReal fakes goods bads perfect ccFixups real insts receiverClasses |

	mapFakeClassesToReal _ smartRefStream reshapedClassesIn: outPointers.
		"Dictionary of just the ones that change shape.  Substitute them in outPointers."
	ccFixups _ self remapCompactClasses: mapFakeClassesToReal 
				refStrm: smartRefStream.
	ccFixups ifFalse: [^ self error: 'A class in the file is not compatible'].
	endMarker _ segment nextObject. 	"for enumeration of objects"
	endMarker == 0 ifTrue: [endMarker _ 'End' clone].
	arrayOfRoots _ self loadSegmentFrom: segment outPointers: outPointers.
		"Can't use install.  Not ready for rehashSets"
	mapFakeClassesToReal isEmpty ifFalse: [
		fakes _ mapFakeClassesToReal keys.
		goods _ OrderedCollection new.
		bads _ OrderedCollection new.
		fakes do: [:aFakeClass | 
			real _ mapFakeClassesToReal at: aFakeClass.
			(real indexIfCompact > 0) "and there is a fake class"
				ifFalse: ["normal case"
					aFakeClass allInstancesDo: [:misShapen | 
						perfect _ smartRefStream convert: misShapen to: real.
						(bads includes: misShapen) ifFalse: [
							bads add: misShapen.
							goods add: perfect]]]
				ifTrue: ["instances have the wrong class.  Fix them before anyone notices."
					insts _ OrderedCollection new.
					self allObjectsDo: [:obj | obj class == real ifTrue: [insts add: obj]].
					insts do: [:misShapen | 
						perfect _ smartRefStream convert: misShapen to: real.
						(bads includes: misShapen) ifFalse: [
							bads add: misShapen.
							goods add: perfect]]]].

		bads size > 0 ifTrue: [
			bads asArray elementsForwardIdentityTo: goods asArray]].
	receiverClasses _ self restoreEndianness.		"rehash sets"
	smartRefStream checkFatalReshape: receiverClasses.

	"Classes in this segment."
	arrayOfRoots do: [:possibleProject | 
		(possibleProject isKindOf: Project) ifTrue: [
			possibleProject ensureChangeSetNameUnique.
			Project addingProject: possibleProject.
			possibleProject restoreReferences].
		possibleProject class class == Metaclass ifTrue: [self declare: possibleProject]].

	mapFakeClassesToReal isEmpty ifFalse: [
		fakes do: [:aFake | 
			aFake indexIfCompact > 0 ifTrue: [aFake becomeUncompact].
			aFake removeFromSystemUnlogged].
		SystemOrganization removeEmptyCategories].
	"^ self"! !


!PhraseTileMorph methodsFor: 'mouse' stamp: 'tk 12/6/2000 11:47'!
morphToDropInPasteUp: aPasteUp
	"Answer the morph to drop in aPasteUp, given that the receiver is the putative droppee"

	| actualObject itsSelector aScriptor anEditor adjustment handy |

	self isCommand ifFalse: [^ self].
	(actualObject _ self actualObject) ifNil: [^ self].
	actualObject assureUniClass.

	itsSelector _ self userScriptSelector.
	aScriptor _ (itsSelector ~~ nil and: [itsSelector size > 0])
		ifTrue:
			[actualObject isFlagshipForClass
				ifFalse: 
					["We can set the status for our instantiation of this script, but cannot allow script editing"
					anEditor _ actualObject scriptEvaluatorFor: itsSelector phrase: self.
					adjustment _ 50 @ 40.
					anEditor]
				ifTrue:
					[(actualObject class userScriptForPlayer: actualObject selector: itsSelector) 
						isTextuallyCoded ifTrue:
							[^ aPasteUp scriptorForTextualScript: itsSelector ofPlayer: actualObject].
					adjustment _ 0@0.
					aScriptor _ actualObject scriptEditorFor: itsSelector]]
		ifFalse:
			["It's a system-defined selector; construct an anonymous scriptor around it"
			adjustment _ 60 @ 20.
			actualObject newScriptorAround: self].

	handy _ aPasteUp primaryHand.
	aScriptor ifNotNil: [aScriptor position: handy position - adjustment].
	(aScriptor isKindOf: ScriptEditorMorph) ifTrue:
		[aScriptor playerScripted expungeEmptyUnRenamedScripts].
	^ aScriptor ifNil: [self]! !


!Project methodsFor: 'file in/out' stamp: 'tk 12/5/2000 09:46'!
exportSegmentWithCatagories: catList classes: classList fileName: aFileName directory: aDirectory
	"Store my project out on the disk as an *exported* ImageSegment.  All outPointers will be in a form that can be resolved in the target image.  Name it <project name>.extSeg.  What do we do about subProjects, especially if they are out as local image segments?  Force them to come in?
	Player classes are included automatically."

| is str ans revertSeg roots holder |
"world == World ifTrue: [^ false]."
	"self inform: 'Can''t send the current world out'."
world isMorph ifFalse: [
	self projectParameters at: #isMVC put: true.
	^ false].	"Only Morphic projects for now"
world ifNil: [^ false].  world presenter ifNil: [^ false].

Utilities emptyScrapsBook.
world currentHand pasteBuffer: nil.	  "don't write the paste buffer."
	"Display checkCurrentHandForObjectToPaste."
Command initialize.
world clearCommandHistory.
world fullReleaseCachedState. 
world cleanseStepList.
world localFlapTabs size = world flapTabs size ifFalse: [
	self error: 'Still holding onto Global flaps'].
world releaseSqueakPages.
holder _ Project allProjects.	"force them in to outPointers, where DiskProxys are made"

"Just export me, not my previous version"
revertSeg _ self projectParameters at: #revertToMe ifAbsent: [nil].
self projectParameters removeKey: #revertToMe ifAbsent: [].

roots _ OrderedCollection new.
roots add: self; add: world; add: transcript; add: changeSet; add: thumbnail.
roots add: world activeHand; addAll: classList; addAll: (classList collect: [:cls | cls class]).

roots _ roots reject: [ :x | x isNil].	"early saves may not have active hand or thumbnail"

catList do: [:sysCat | 
	(SystemOrganization listAtCategoryNamed: sysCat asSymbol) do: [:symb |
		roots add: (Smalltalk at: symb); add: (Smalltalk at: symb) class]].

is _ ImageSegment new copySmartRootsExport: roots asArray.
	"old way was (is _ ImageSegment new copyFromRootsForExport: roots asArray)"

is state = #tooBig ifTrue: [^ false].

str _ ''.
"considered legal to save a project that has never been entered"
(is outPointers includes: world) ifTrue: [
	str _ str, '\Project''s own world is not in the segment.' withCRs].
str isEmpty ifFalse: [
	ans _ (PopUpMenu labels: 'Do not write file
Write file anyway
Debug') startUpWithCaption: str.
	ans = 1 ifTrue: [
		revertSeg ifNotNil: [projectParameters at: #revertToMe put: revertSeg].
		^ false].
	ans = 3 ifTrue: [self halt: 'Segment not written']].

is writeForExportWithSources: aFileName inDirectory: aDirectory.
revertSeg ifNotNil: [projectParameters at: #revertToMe put: revertSeg].
holder.
^ true
! !

!Project methodsFor: 'file in/out' stamp: 'tk 12/7/2000 12:13'!
restoreReferences
	| key newKey |
	"I just came in from an exported segment.  Take all my players that were in References, and reinstall them."

	(world valueOfProperty: #References ifAbsent: [#()]) do: [:assoc |
		key _ assoc key.
		(References includesKey: key) 
			ifTrue: ["must rename" 
				(References at: key) == assoc value ifTrue: [
					self error: 'why object already present?'].
				newKey _ assoc value uniqueNameForReference.
				References removeKey: newKey.
				assoc key: newKey.
				References add: assoc.	"use the known association"
				assoc value renameTo: newKey.
				]
			ifFalse: [References add: assoc]].
	world removeProperty: #References.! !


!SequenceableCollection methodsFor: 'comparing' stamp: 'tk 12/6/2000 11:39'!
hasEqualElements: otherCollection
	"Answer whether the receiver's size is the same as otherCollection's
	size, and each of the receiver's elements equal the corresponding 
	element of otherCollection.
	This should probably replace the current definition of #= ."

	| size |
	(otherCollection isKindOf: SequenceableCollection) ifFalse: [^ false].
	(size _ self size) = otherCollection size ifFalse: [^ false].
	1 to: size do:
		[:index |
		(self at: index) = (otherCollection at: index) ifFalse: [^ false]].
	^ true! !


!SyntaxMorph methodsFor: 'dropping/grabbing' stamp: 'tk 12/6/2000 11:48'!
morphToDropInPasteUp: aPasteUp
	| actualObject itsSelector aScriptor anEditor adjustment handy tw blk |
	"If property #beScript is true, create a scriptor around me."

	(self valueOfProperty: #beScript ifAbsent: [false]) ifFalse: [^ self].
	self removeProperty: #beScript.

	(actualObject _ self actualObject) ifNil: [^ self].
	actualObject assureUniClass.

	itsSelector _ self userScriptSelector.
	aScriptor _ (itsSelector ~~ nil and: [itsSelector size > 0])
		ifTrue:	"user script"
			[actualObject isFlagshipForClass
				ifFalse: 
					["We can set the status for our instantiation of this script, but cannot allow script editing"
					anEditor _ actualObject scriptEvaluatorFor: itsSelector phrase: self.
					adjustment _ 50 @ 40.
					anEditor]
				ifTrue:
					[(aPasteUp world valueOfProperty: #universalTiles ifAbsent: [false]) not &
					(actualObject class userScriptForPlayer: actualObject selector: itsSelector) 
						isTextuallyCoded ifTrue:
							[^ aPasteUp scriptorForTextualScript: itsSelector ofPlayer: actualObject].
					adjustment _ 0@0.
					aScriptor _ actualObject scriptEditorFor: itsSelector]]
		ifFalse:
			["It's a system-defined selector; construct an anonymous scriptor around it"
			itsSelector _ nil.
			adjustment _ 60 @ 20.
			actualObject newScriptorAround: self].

	handy _ aPasteUp primaryHand.
	aScriptor ifNotNil: [
		aScriptor position: handy position - adjustment.
		aPasteUp addMorphFront: aScriptor.	"do this early so can find World"
		(tw _ aScriptor findA: TwoWayScrollPane) ifNil: [
			aScriptor useNewTiles.
			itsSelector ifNil: ["blank script"
				tw _ aScriptor findA: TwoWayScrollPane.
				blk _ tw scroller firstSubmorph "MethodNode" lastSubmorph "BlockNode".
				blk addMorphFront: self.
				"self position: self topLeft + (7@14)"]]].
	aScriptor showSourceInScriptor. 
	(aScriptor isKindOf: ScriptEditorMorph) ifTrue:
		[aScriptor playerScripted expungeEmptyUnRenamedScripts].
	^ aScriptor ifNil: [self]
! !

