'From Squeak 2.2 of Sept 23, 1998 on 25 November 1998 at 2:44:28 pm'!
"Change Set:		ObjectMemTweaks
Date:			23 November 1998
Author:			John Maloney

Some VM improvements from Georg Gollmann:
  - fixed a bug that limited image size to about 512 MB
  - fixed a bug that limited object size to below 256 MB
  - changed some literal constants to symbolic ones

Modified by John Maloney to:
  - merge with WeakArray support
  - omitted the change allowing use of the upper half of the
    address space since it made incremental GC slightly slower (1-2%)
    and does not seem likely to be needed soon (few machines have
    over 2 gigabytes of memory!!!!)"
!

Object subclass: #ObjectMemory
	instanceVariableNames: 'memory youngStart endOfMemory memoryLimit nilObj falseObj trueObj specialObjectsOop rootTable rootTableCount child field parentField freeBlock lastHash freeLargeContexts freeSmallContexts allocationCount lowSpaceThreshold signalLowSpace compStart compEnd fwdTableNext fwdTableLast remapBuffer remapBufferCount interruptCheckCounter checkAssertions allocationsBetweenGCs tenuringThreshold statFullGCs statFullGCMSecs statIncrGCs statIncrGCMSecs statTenures statRootTableOverflows displayBits '
	classVariableNames: 'AllButHashBits AllButMarkBit AllButMarkBitAndTypeMask AllButRootBit AllButTypeMask BaseHeaderSize CharacterTable ClassArray ClassBitmap ClassBlockContext ClassByteArray ClassCharacter ClassCompiledMethod ClassFloat ClassInteger ClassLargePositiveInteger ClassMessage ClassMethodContext ClassPoint ClassProcess ClassPseudoContext ClassSemaphore ClassString ClassTranslatedMethod CompactClassMask CompactClasses ConstMinusOne ConstOne ConstTwo ConstZero Done ExternalObjectsArray FalseObject FloatProto FreeSizeMask GCTopMarker HashBits HashBitsOffset HeaderTypeClass HeaderTypeFree HeaderTypeGC HeaderTypeShort HeaderTypeSizeAndClass LargeContextSize MarkBit MinimumForwardTableBytes NilContext NilObject RemapBufferSize RootBit RootTableSize SchedulerAssociation SelectorCannotReturn SelectorDoesNotUnderstand SelectorMustBeBoolean SizeMask SmallBlockContext SmallContextSize SmallMethodContext SpecialSelectors StackStart StartField StartObj TheDisplay TheFinalizationSemaphore TheInputSemaphore TheInterruptSemaphore TheLowSpaceSemaphore TheTimerSemaphore TrueObject TypeMask Upward '
	poolDictionaries: ''
	category: 'Squeak-Interpreter'!

!ObjectMemory methodsFor: 'interpreter access' stamp: 'jm 11/24/1998 00:13'!
instantiateClass: classPointer indexableSize: size
	| hash header1 header2 cClass byteSize format inc binc header3 hdrSize fillWord newObj sizeHiBits |
"
	NOTE: This method supports the backward-compatible split instSize field of the
	class format word.  The sizeHiBits will go away and other shifts change by 2
	when the split fields get merged in an (incompatible) image change.
"
	self inline: false.
	checkAssertions ifTrue: [
		size < 0 ifTrue: [ self error: 'cannot have a negative indexable field count' ]].

	hash _ self newObjectHash.
	header1 _ self formatOfClass: classPointer. "Low 2 bits are 0"
	sizeHiBits _ (header1 bitAnd: 16r60000) >> 9.
	header1 _ (header1 bitAnd: 16r1FFFF) bitOr: ((hash << HashBitsOffset) bitAnd: HashBits).
	header2 _ classPointer.
	header3 _ 0.

	cClass _ header1 bitAnd: CompactClassMask. "compact class field from format word"
	byteSize _ (header1 bitAnd: SizeMask) + sizeHiBits. "size in bytes -- low 2 bits are 0"
	format _ (header1 >> 8) bitAnd: 16rF.

	format < 8 ifTrue: [
		"Bitmaps and Arrays"
		inc _ size * 4.
	] ifFalse: [
		"Strings and Methods"
		inc _ (size + 3) bitAnd: AllButTypeMask. "round up"
		binc _ 3 - ((size + 3) bitAnd: 3). "odd bytes"
		"low bits of byte size go in format field"
		header1 _ header1 bitOr: (binc << 8).
	].

	(byteSize + inc) > 255 ifTrue: [
		"requires size header word"
		header3 _ byteSize + inc.
		header1 _ header1 - (byteSize bitAnd: 16rFF).  "Clear qsize field"
	] ifFalse: [
		header1 _ header1 + inc.
	].
	byteSize _ byteSize + inc.

	header3 > 0 ifTrue: [
		"requires full header"
		hdrSize _ 3.
	] ifFalse: [
		cClass = 0
			ifTrue: [ hdrSize _ 2 ]
			ifFalse: [ hdrSize _ 1 ].
	].
	format <= 4  "if pointers, fill with nil oop"
		ifTrue: [ fillWord _ nilObj ]
		ifFalse: [ fillWord _ 0 ].

	newObj _ self allocate: byteSize headerSize: hdrSize h1: header1 h2: header2 h3: header3 fill: fillWord.
	^ newObj! !

!ObjectMemory methodsFor: 'interpreter access' stamp: 'go 11/18/1998 11:03'!
instantiateSmallClass: classPointer sizeInBytes: sizeInBytes fill: fillValue
	"This version of instantiateClass assumes that the total object size is under 256 bytes, the limit for objects with only one or two header words. Note that the size is specified in bytes and should include four bytes for the base header word."

	| hash header1 header2 hdrSize |
	hash _ self newObjectHash.
	header1 _ ((hash << HashBitsOffset) bitAnd: HashBits) bitOr:
			   (self formatOfClass: classPointer).
	header1 _ header1 + (sizeInBytes - (header1 bitAnd: SizeMask)).
	header2 _ classPointer.

	(header1 bitAnd: CompactClassMask) = 0 "is compact class field from format word zero?"
		ifTrue: [ hdrSize _ 2 ]
		ifFalse: [ hdrSize _ 1 ].

	^ self allocate: sizeInBytes headerSize: hdrSize h1: header1 h2: header2 h3: 0 fill: fillValue! !

!ObjectMemory methodsFor: 'header access' stamp: 'go 11/18/1998 11:04'!
rightType: headerWord
	"Computer the correct header type for an object based on the size and compact class fields of the given base header word, rather than its type bits. This is used during marking, when the header type bits are used to record the state of tracing."

	(headerWord bitAnd: SizeMask) = 0  "zero size field in header word"
		ifTrue: [ ^ HeaderTypeSizeAndClass ]
		ifFalse: [
			(headerWord bitAnd: CompactClassMask) = 0
				ifTrue: [ ^ HeaderTypeClass ]
				ifFalse: [ ^ HeaderTypeShort ]].! !

!ObjectMemory methodsFor: 'header access' stamp: 'go 11/13/1998 17:04'!
setSizeOfFree: chunk to: byteSize
	"Set the header of the given chunk to make it be a free chunk of the given size."

	self longAt: chunk put: ((byteSize bitAnd: AllButTypeMask) bitOr: HeaderTypeFree).! !

!ObjectMemory methodsFor: 'header access' stamp: 'go 11/17/1998 15:57'!
sizeBitsOf: oop
	"Answer the number of bytes in the given object, including its base header, rounded up to an integral number of words."
	"Note: byte indexable objects need to have low bits subtracted from this size."

	| header |
	header _ self baseHeader: oop.
	(header bitAnd: TypeMask) = HeaderTypeSizeAndClass
		ifTrue: [ ^ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ ^ header bitAnd: SizeMask ].! !

!ObjectMemory methodsFor: 'header access' stamp: 'go 11/17/1998 15:57'!
sizeBitsOfSafe: oop
	"Compute the size of the given object from the cc and size fields in its header. This works even if its type bits are not correct."

	| header type |
	header _ self baseHeader: oop.
	type _ self rightType: header.
	type = HeaderTypeSizeAndClass
		ifTrue: [ ^ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ ^ header bitAnd: SizeMask ].! !

!ObjectMemory methodsFor: 'header access' stamp: 'go 11/13/1998 17:04'!
sizeOfFree: oop
	"Return the size of the given chunk in bytes. Argument MUST be a free chunk."

	^ (self longAt: oop) bitAnd: AllButTypeMask! !

!ObjectMemory methodsFor: 'allocation' stamp: 'go 11/18/1998 11:04'!
recycleContextIfPossible: cntxOop methodContextClass: methodCntxClass
	"If possible, save the given context on a list of free contexts to be recycled."
	"Note: The context is not marked free, so it can be reused with minimal fuss. It's fields are nil-ed out when it is re-used. The recycled context lists are cleared at every garbage collect."
	"Note: This code was found to be critical to good send/return speed, so it has been ruthlessly hand-tuned."

	| cntxHeader ccField isMethodCntx |
	self inline: true.
	"only recycle young contexts (which should be most of them)"
	cntxOop >= youngStart ifTrue: [
		"is the context of class methodCntxClass?"
		cntxHeader _ self baseHeader: cntxOop.
		ccField _ cntxHeader bitAnd: CompactClassMask.
		ccField = 0 ifTrue: [
			isMethodCntx _ ((self classHeader: cntxOop) bitAnd: AllButTypeMask) = methodCntxClass.
		] ifFalse: [
			"compare ccField with compact class bits from format word of methodCntxClass"
			isMethodCntx _ ccField = ((self formatOfClass: methodCntxClass) bitAnd: CompactClassMask).
		].

		isMethodCntx ifTrue: [
			"Note: The following test depends on the format of object headers
			 and the fact that both small and large contexts are small enough
			 for their size to be encoded in the base object header. If these
			 assumptions is false, contexts won't be recycled properly, but the
			 code should not break."

			(cntxHeader bitAnd: SizeMask) = SmallContextSize ifTrue: [
				self storePointerUnchecked: 0 ofObject: cntxOop withValue: freeSmallContexts.
				freeSmallContexts _ cntxOop.	
			] ifFalse: [
				self storePointerUnchecked: 0 ofObject: cntxOop withValue: freeLargeContexts.
				freeLargeContexts _ cntxOop.	
			].
		].
	].
! !

!ObjectMemory methodsFor: 'garbage collection' stamp: 'go 11/17/1998 15:56'!
lowestFreeAfter: chunk
	"Return the first free block after the given chunk in memory."

	| oop oopHeader oopHeaderType oopSize |
	self inline: false.
	oop _ self oopFromChunk: chunk.
	[oop < endOfMemory] whileTrue: [
		oopHeader _ self baseHeader: oop.
		oopHeaderType _ oopHeader bitAnd: TypeMask.
		(oopHeaderType = HeaderTypeFree)
			ifTrue: [ ^ oop ]
			ifFalse: [
				oopHeaderType = HeaderTypeSizeAndClass
					ifTrue: [ oopSize _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
					ifFalse: [ oopSize _ oopHeader bitAnd: SizeMask ].
			].
		oop _ self oopFromChunk: (oop + oopSize).
	].
	self error: 'expected to find at least one free object'.
! !

!ObjectMemory methodsFor: 'gc -- mark and sweep' stamp: 'go 11/18/1998 11:04'!
startField
	"Examine and possibly trace the next field of the object being traced. See comment in markAndTrace for explanation of tracer state variables."

	| typeBits childType |
	child _ self longAt: field.
	typeBits _ child bitAnd: TypeMask.

	(typeBits bitAnd: 1) = 1 ifTrue: [
		"field contains a SmallInteger; skip it"
		field _ field - 4.
		^ StartField
	].

	typeBits = 0 ifTrue: [
		"normal oop, go down"
		self longAt: field put: parentField.
		parentField _ field.
		^ StartObj
	].

	typeBits = 2 ifTrue: [
		"reached the header; do we need to process the class word?"
		(child bitAnd: CompactClassMask) ~= 0 ifTrue: [
			"object's class is compact; we're done"
			"restore the header type bits"
			child _ child bitAnd: AllButTypeMask.
			childType _ self rightType: child.
			self longAt: field put: (child bitOr: childType).
			^ Upward
		] ifFalse: [
			"object has a full class word; process that class"
			child _ self longAt: (field - 4).  "class word"
			child _ child bitAnd: AllButTypeMask.  "clear type bits"
			self longAt: (field - 4) put: parentField.
			parentField _ (field - 4) bitOr: 1.  "point at class word; mark as working on the class."
			^ StartObj
		].
	].! !

!ObjectMemory methodsFor: 'gc -- mark and sweep' stamp: 'jm 11/23/1998 23:01'!
sweepPhase
	"Sweep memory from youngStart through the end of memory. Free all inaccessible objects and coalesce adjacent free chunks. Clear the mark bits of accessible objects. Compute the starting point for the first pass of incremental compaction (compStart). Return the number of surviving objects."
	"Details: Each time a non-free object is encountered, decrement the number of available forward table entries. If all entries are spoken for (i.e., entriesAvailable reaches zero), set compStart to the last free chunk before that object or, if there is no free chunk before the given object, the first free chunk after it. Thus, at the end of the sweep phase, compStart through compEnd spans the highest collection of non-free objects that can be accomodated by the forwarding table. This information is used by the first pass of incremental compaction to ensure that space is initially freed at the end of memory. Note that there should always be at least one free chunk--the one at the end of the heap."

	| entriesAvailable survivors freeChunk firstFree oop oopHeader oopHeaderType hdrBytes oopSize freeChunkSize |
	self inline: false.
	entriesAvailable _ self fwdTableInit.
	survivors _ 0.
	freeChunk _ nil.
	firstFree _ nil.  "will be updated later"
	oop _ self oopFromChunk: youngStart.
	[oop < endOfMemory] whileTrue: [
		"get oop's header, header type, size, and header size"
		oopHeader _ self baseHeader: oop.
		oopHeaderType _ oopHeader bitAnd: TypeMask.
		(oopHeaderType = HeaderTypeShort) ifTrue: [
			oopSize _ oopHeader bitAnd: SizeMask.
			hdrBytes _ 0.
		] ifFalse: [
			(oopHeaderType = HeaderTypeClass) ifTrue: [
				oopSize _ oopHeader bitAnd: SizeMask.
				hdrBytes _ 4.
			] ifFalse: [
				(oopHeaderType = HeaderTypeSizeAndClass) ifTrue: [
					oopSize _ (self sizeHeader: oop) bitAnd: AllButTypeMask.
					hdrBytes _ 8.
				] ifFalse: [  "free chunk"
					oopSize _ oopHeader bitAnd: AllButTypeMask.
					hdrBytes _ 0.
				].
			].
		].

		(oopHeader bitAnd: MarkBit) = 0 ifTrue: ["object is not marked; free it"
			"<-- Finalization support: We need to mark each oop chunk as free -->"
			self longAt: oop - hdrBytes put: HeaderTypeFree.
			freeChunk ~= nil ifTrue: [
				"enlarge current free chunk to include this oop"
				freeChunkSize _ freeChunkSize + oopSize + hdrBytes.
			] ifFalse: [
				"start a new free chunk"
				freeChunk _ oop - hdrBytes.  "chunk may start 4 or 8 bytes before oop"
				freeChunkSize _ oopSize + (oop - freeChunk).  "adjust size for possible extra header bytes"
				firstFree = nil ifTrue: [ firstFree _ freeChunk ].
			].
		] ifFalse: [
			"object is marked; clear its mark bit and possibly adjust the compaction start"
			self longAt: oop put: (oopHeader bitAnd: AllButMarkBit).
			"<-- Finalization support: Check if we're running about a weak class -->"
			(self isWeak: oop) ifTrue:[
				self finalizeReference: oop.
			].
			entriesAvailable > 0 ifTrue: [
				entriesAvailable _ entriesAvailable - 1.
			] ifFalse: [
				"start compaction at the last free chunk before this object"
				firstFree _ freeChunk.
			].
			freeChunk ~= nil ifTrue: [
				"record the size of the last free chunk"
				self longAt: freeChunk
					    put: ((freeChunkSize bitAnd: AllButTypeMask) bitOr: HeaderTypeFree).
			].
			freeChunk _ nil.
			survivors _ survivors + 1.
		].
		oop _ self oopFromChunk: (oop + oopSize).  "get next oop"
	].

	freeChunk ~= nil ifTrue: [
		"record size of final free chunk"
		self longAt: freeChunk
			    put: ((freeChunkSize bitAnd: AllButTypeMask) bitOr: HeaderTypeFree).
	].

	oop = endOfMemory
		ifFalse: [ self error: 'sweep failed to find exact end of memory' ].

	firstFree = nil
		ifTrue: [ self error: 'expected to find at least one free object' ]
		ifFalse: [ compStart _ firstFree ].

	displayBits = 0 ifFalse: [
		"TPR: clear mark bit of Acorn's displayBits object, which may lie outside object space"
		oopHeader _ self baseHeader: displayBits.
		self longAt: displayBits put: (oopHeader bitAnd: AllButMarkBit)].	

	^ survivors
! !

!ObjectMemory methodsFor: 'gc -- compaction' stamp: 'jm 11/24/1998 00:07'!
fwdTableInit
	"Set the limits for a table of two-word forwarding blocks above the last used oop. The pointer fwdTableNext moves up to fwdTableLast. Used for compaction of memory and become-ing objects. Returns the number of forwarding blocks available."

	self inline: false.
	"set endOfMemory to just after a minimum-sized free block"
	self setSizeOfFree: freeBlock to: BaseHeaderSize.
	endOfMemory _ freeBlock + BaseHeaderSize.

	"make a fake free chunk at endOfMemory for use as a sentinal in memory scans"
	self setSizeOfFree: endOfMemory to: BaseHeaderSize.

	"use all memory free between freeBlock and memoryLimit for forwarding table"
	"(use quadword alignment for better cache performance)"
	fwdTableNext _ (endOfMemory + BaseHeaderSize + 7) bitAnd: 16rFFFFFFF8.
	fwdTableLast _ memoryLimit - 8.  "last forwarding table entry"

	(checkAssertions and: [(fwdTableLast bitAnd: MarkBit) ~= 0]) ifTrue: [
		"Note: Address bits must not interfere with the mark bit in header of
		 an object, which shows that the object is forwarded."
		self error: 'fwd table must be in low half of the 32-bit address space'.
	].

	"return the number of forwarding blocks available"
	^ (fwdTableLast - fwdTableNext) // 8  "round down"! !

!ObjectMemory methodsFor: 'gc -- compaction' stamp: 'jm 11/24/1998 00:03'!
lastPointerWhileForwarding: oop
	"The given object may have its header word in a forwarding block. Find the offset of the last pointer in the object in spite of this obstacle."

	| header fwdBlock fmt size methodHeader |
	self inline: true.
	header _ self longAt: oop.
	(header bitAnd: MarkBit) ~= 0 ifTrue: [
		"oop is forwarded; get its real header from its forwarding table entry"
		fwdBlock _ header bitAnd: AllButMarkBitAndTypeMask.
		checkAssertions ifTrue: [ self fwdBlockValidate: fwdBlock ].
		header _ self longAt: fwdBlock + 4.
	].

	fmt _ (header >> 8) bitAnd: 16rF.
	fmt <= 4 ifTrue: [
		"do sizeBitsOf: using the header we obtained"
		(header bitAnd: TypeMask) = HeaderTypeSizeAndClass
			ifTrue: [ size _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
			ifFalse: [ size _ header bitAnd: SizeMask ].
		^ size - BaseHeaderSize
	].
	fmt < 12 ifTrue: [ ^ 0 ].  "no pointers"

	methodHeader _ self longAt: oop + BaseHeaderSize.
	^ ((methodHeader >> 10) bitAnd: 16rFF) * 4 + BaseHeaderSize! !

!ObjectMemory methodsFor: 'gc -- compaction' stamp: 'jm 11/23/1998 23:12'!
objectAfterWhileForwarding: oop
	"Return the oop of the object after the given oop when the actual header of the oop may be in the forwarding table."

	| header fwdBlock realHeader sz |
	self inline: true.
	header _ self longAt: oop.
	(header bitAnd: MarkBit) = 0 ifTrue: [ ^ self objectAfter: oop ].  "oop not forwarded"

	"Assume: mark bit cannot be set on a free chunk, so if we get here,
	 oop is not free and it has a forwarding table entry"

	fwdBlock _ header bitAnd: AllButMarkBitAndTypeMask.
	checkAssertions ifTrue: [ self fwdBlockValidate: fwdBlock ].
	realHeader _ self longAt: fwdBlock + 4.
	"following code is like sizeBitsOf:"
	(realHeader bitAnd: TypeMask) = HeaderTypeSizeAndClass
		ifTrue: [ sz _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ sz _ realHeader bitAnd: SizeMask ].

	^ self oopFromChunk: (oop + sz)! !


!DynamicContextCache methodsFor: 'utilities' stamp: 'go 11/18/1998 13:00'!
changeClassOf: anObject to: aClass
	"Used to mutate a PseudoContext to/from a stable Method/BlockContext.
	Notes:	Tacitly assumes that the type bits are the SAME for the source and destination!!
			This method should be in ObjectMemory."

	| ccClass hdrObject ccObject |
	self inline: false.
	hdrObject _ self baseHeader: anObject.
	ccObject _ hdrObject bitAnd: CompactClassMask.
	ccObject = 0 ifTrue: [
		"object has uncompact class"
		self classHeader: anObject put: aClass.
	] ifFalse: [
		"object has compact class"
		ccClass _ (self formatOfClass: aClass) bitAnd: CompactClassMask.
		ccClass ~= 0 ifTrue: [
			"object has compact class; class is compact"
			hdrObject _ (hdrObject bitXor: ccObject) bitOr: ccClass.
			self baseHeader: anObject put: hdrObject.
		] ifFalse: [
			"object has compact class; class is uncompact"
			self error: 'cannot mutate header from compact to uncompact'.
		]
	]! !


!DynamicTranslator methodsFor: '  fake inline cache' stamp: 'go 11/18/1998 10:51'!
compactSend: tMeth type: type

	| nArgs rcvr rcvrClass |
	self inline: true.

	self assertIsTranslatedMethod: tMeth.
	nArgs _ self fetchPointer: MethodArgCountIndex ofObject: tMeth.
	nArgs _ self integerValueOf: nArgs.
	rcvr _ self internalStackValue: nArgs.
	(self isIntegerObject: rcvr)
		ifTrue: [rcvrClass _ 0]
		ifFalse: [rcvrClass _ ((self baseHeader: rcvr) bitAnd: CompactClassMask) + 1].
	self checkSend: tMeth to: rcvr data: rcvrClass nArgs: nArgs type: type! !

!DynamicTranslator methodsFor: '  fake inline cache' stamp: 'go 11/18/1998 10:51'!
linkSendOpcode: ip type: type receiver: rcvr class: rcvrClass method: tMeth
	"Rewrite the send opcode at ip.
	ip:		the address of the linked extension word.
	type:	0=short, 1=extended, 2=double-extended.
	class:	the receiver class for which the inline cache hits."

	"Note: the opcode itself is located at (ip - 4 - (type * 8))."

	| op addr sendType linkData ccIndex currentMethod |

	self assertAny: (addr _ 0).
	self assertAny: (sendType _ 0).

	(self isIntegerObject: rcvr)
		ifTrue: [sendType _ ImmediateSendType.  linkData _ ConstZero]
		ifFalse:
			[ccIndex _ ((self formatOfClass: rcvrClass) bitAnd: CompactClassMask).
			 ccIndex > 0
				ifTrue: [sendType _ CompactSendType.  linkData _ ccIndex + 1]
				ifFalse: [sendType _ NormalSendType.  linkData _ rcvrClass]].

	self assert: (sendType ~= 0).

	type = ShortSendType	 		ifTrue:	[addr _ ip - 4.	op _ shortSendTable at: sendType]			ifFalse: [
	type = ExtendedSendType			ifTrue:	[addr _ ip - 12.	op _ extendedSendTable at: sendType]		ifFalse: [
	type = DoubleExtendedSendType	ifTrue:	[addr _ ip - 20.	op _ doubleExtendedSendTable at: sendType]	]].

	self assert: (addr ~= 0).

	self longAt: addr put: op.
	self longAt: ip put: tMeth.
	self storePointer: MethodLinkageIndex ofObject: tMeth withValue: linkData.

	tMeth >= youngStart ifTrue:		"store of young object into old object"
		[currentMethod _ self translatedMethod.
		 currentMethod < youngStart
			ifTrue: [self addMethodRoot: currentMethod]].
! !

!DynamicTranslator methodsFor: 'obsolete' stamp: 'go 11/18/1998 10:51'!
oldCompactSend: tMeth type: type

	| nArgs rcvr prevClass rcvrClass |
	self inline: true.

	self assertIsTranslatedMethod: tMeth.
	nArgs _ self fetchPointer: MethodArgCountIndex ofObject: tMeth.
	nArgs _ self integerValueOf: nArgs.
	rcvr _ self internalStackValue: nArgs.
	prevClass _ (self fetchPointer: MethodLinkageIndex ofObject: tMeth) - 1.
	self assert: prevClass ~= 0.
	(self isIntegerObject: rcvr)
		ifTrue: [rcvrClass _ 0]
		ifFalse: [rcvrClass _ ((self baseHeader: rcvr) bitAnd: CompactClassMask)].
	(rcvrClass = prevClass) ifTrue: [
		self executeLinkedSend: tMeth to: rcvr nArgs: nArgs.
	] ifFalse: [
		self relinkSend: tMeth to: rcvr nArgs: nArgs type: type.
	]! !


!DynamicInterpreter methodsFor: 'array and stream primitives' stamp: 'go 11/17/1998 15:54'!
byteLengthOf: oop
	"Return the number of indexable bytes in the given object. This is basically a special copy of lengthOf: for BitBlt."
	| header sz fmt |
	header _ self baseHeader: oop.
	(header bitAnd: TypeMask) = HeaderTypeSizeAndClass
		ifTrue: [ sz _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ sz _ header bitAnd: SizeMask ].
	fmt _ (header >> 8) bitAnd: 16rF.
	fmt < 8
		ifTrue: [ ^ (sz - BaseHeaderSize)]  "words"
		ifFalse: [ ^ (sz - BaseHeaderSize) - (fmt bitAnd: 3)]  "bytes"! !

!DynamicInterpreter methodsFor: 'array and stream primitives' stamp: 'go 11/17/1998 15:54'!
lengthOf: oop
	"Return the number of indexable bytes or words in the given object. Assume the argument is not an integer. For a CompiledMethod, the size of the method header (in bytes) should be subtracted from the result."

	| header sz fmt |
	self inline: true.
	"from ObjectMemory>sizeBitsOf:..."
	header _ self baseHeader: oop.
	(header bitAnd: TypeMask) = HeaderTypeSizeAndClass
		ifTrue: [ sz _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ sz _ header bitAnd: SizeMask ].

	"from ObjectMemory>formatOf:..."
	fmt _ (header >> 8) bitAnd: 16rF.

	fmt < 8
		ifTrue: [ ^ (sz - BaseHeaderSize) >> 2 ]  "words"
		ifFalse: [ ^ (sz - BaseHeaderSize) - (fmt bitAnd: 3) ]  "bytes"! !

!DynamicInterpreter methodsFor: 'array and stream primitives' stamp: 'go 11/17/1998 15:54'!
lengthOf: oop baseHeader: hdr format: fmt
	"Return the number of indexable bytes or words in the given object. Assume the given oop is not an integer. For a CompiledMethod, the size of the method header (in bytes) should be subtracted from the result of this method."

	| sz |
	self inline: true.
	(hdr bitAnd: TypeMask) = HeaderTypeSizeAndClass
		ifTrue: [ sz _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ sz _ hdr bitAnd: SizeMask ].

	fmt < 8
		ifTrue: [ ^ (sz - BaseHeaderSize) >> 2 ]  "words"
		ifFalse: [ ^ (sz - BaseHeaderSize) - (fmt bitAnd: 3) ]  "bytes"! !


!DynamicInterpreterSimulator methodsFor: 'testing' stamp: 'go 11/18/1998 10:50'!
validateOopsIn: object
	| fieldPtr limit former header | 
	"for each oop in me see if it is legal"
	fieldPtr _ object + BaseHeaderSize.	"first field"
	limit _ object + (self lastPointerOf: object).	"a good field"
	[fieldPtr > limit] whileFalse: [
		former _ self longAt: fieldPtr.
		self validOop: former.
		fieldPtr _ fieldPtr + 4].
	"class"
	header _ self baseHeader: object.
	(header bitAnd: CompactClassMask) = 0 ifTrue: [	
		former _ (self classHeader: object) bitAnd: AllButTypeMask.
		self validOop: former].! !

!DynamicInterpreterSimulator methodsFor: 'debug support' stamp: 'jm 11/23/1998 22:44'!
dumpHeader: hdr
	| cc |
	^ String streamContents: [:strm |
		cc _ (hdr bitAnd: CompactClassMask) >> 12.
		strm nextPutAll: '<cc=', cc hex.
		cc > 0 ifTrue:
			[strm nextPutAll: ':' , (self nameOfClass: (self compactClassAt: cc))].
		strm nextPutAll: '>'.
		strm nextPutAll: '<ft=', ((hdr bitShift: -8) bitAnd: 16rF) hex , '>'.
		strm nextPutAll: '<sz=', (hdr bitAnd: SizeMask) hex , '>'.
		strm nextPutAll: '<hdr=', (#(big class gcMark short) at: (hdr bitAnd: 3) +1) , '>']
! !


!Interpreter methodsFor: 'array and stream primitives' stamp: 'go 11/17/1998 15:55'!
byteLengthOf: oop
	"Return the number of indexable bytes in the given object. This is basically a special copy of lengthOf: for BitBlt."
	| header sz fmt |
	header _ self baseHeader: oop.
	(header bitAnd: TypeMask) = HeaderTypeSizeAndClass
		ifTrue: [ sz _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ sz _ header bitAnd: SizeMask ].
	fmt _ (header >> 8) bitAnd: 16rF.
	fmt < 8
		ifTrue: [ ^ (sz - BaseHeaderSize)]  "words"
		ifFalse: [ ^ (sz - BaseHeaderSize) - (fmt bitAnd: 3)]  "bytes"! !

!Interpreter methodsFor: 'array and stream primitives' stamp: 'go 11/17/1998 15:55'!
lengthOf: oop
	"Return the number of indexable bytes or words in the given object. Assume the argument is not an integer. For a CompiledMethod, the size of the method header (in bytes) should be subtracted from the result."

	| header sz fmt |
	self inline: true.
	"from ObjectMemory>sizeBitsOf:..."
	header _ self baseHeader: oop.
	(header bitAnd: TypeMask) = HeaderTypeSizeAndClass
		ifTrue: [ sz _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ sz _ header bitAnd: SizeMask ].

	"from ObjectMemory>formatOf:..."
	fmt _ (header >> 8) bitAnd: 16rF.

	fmt < 8
		ifTrue: [ ^ (sz - BaseHeaderSize) >> 2 ]  "words"
		ifFalse: [ ^ (sz - BaseHeaderSize) - (fmt bitAnd: 3) ]  "bytes"! !

!Interpreter methodsFor: 'array and stream primitives' stamp: 'go 11/17/1998 15:55'!
lengthOf: oop baseHeader: hdr format: fmt
	"Return the number of indexable bytes or words in the given object. Assume the given oop is not an integer. For a CompiledMethod, the size of the method header (in bytes) should be subtracted from the result of this method."

	| sz |
	self inline: true.
	(hdr bitAnd: TypeMask) = HeaderTypeSizeAndClass
		ifTrue: [ sz _ (self sizeHeader: oop) bitAnd: AllButTypeMask ]
		ifFalse: [ sz _ hdr bitAnd: SizeMask ].

	fmt < 8
		ifTrue: [ ^ (sz - BaseHeaderSize) >> 2 ]  "words"
		ifFalse: [ ^ (sz - BaseHeaderSize) - (fmt bitAnd: 3) ]  "bytes"! !


!InterpreterSimulator methodsFor: 'testing' stamp: 'go 11/18/1998 10:51'!
validateOopsIn: object
	| fieldPtr limit former header | 
	"for each oop in me see if it is legal"
	fieldPtr _ object + BaseHeaderSize.	"first field"
	limit _ object + (self lastPointerOf: object).	"a good field"
	[fieldPtr > limit] whileFalse: [
		former _ self longAt: fieldPtr.
		self validOop: former.
		fieldPtr _ fieldPtr + 4].
	"class"
	header _ self baseHeader: object.
	(header bitAnd: CompactClassMask) = 0 ifTrue: [	
		former _ (self classHeader: object) bitAnd: AllButTypeMask.
		self validOop: former].! !

!InterpreterSimulator methodsFor: 'debug support' stamp: 'jm 11/23/1998 22:44'!
dumpHeader: hdr
	| cc |
	^ String streamContents: [:strm |
		cc _ (hdr bitAnd: CompactClassMask) >> 12.
		strm nextPutAll: '<cc=', cc hex.
		cc > 0 ifTrue:
			[strm nextPutAll: ':' , (self nameOfClass: (self compactClassAt: cc))].
		strm nextPutAll: '>'.
		strm nextPutAll: '<ft=', ((hdr bitShift: -8) bitAnd: 16rF) hex , '>'.
		strm nextPutAll: '<sz=', (hdr bitAnd: SizeMask) hex , '>'.
		strm nextPutAll: '<hdr=', (#(big class gcMark short) at: (hdr bitAnd: 3) +1) , '>']
! !


!ObjectMemory class methodsFor: 'initialization' stamp: 'go 11/18/1998 10:42'!
initializeObjectHeaderConstants

	BaseHeaderSize _ 4.

	"masks for type field"
	TypeMask _ 3.
	AllButTypeMask _ 16rFFFFFFFF - TypeMask.

	"type field values"
	HeaderTypeSizeAndClass _ 0.
	HeaderTypeClass _ 1.
	HeaderTypeFree _ 2.
	HeaderTypeShort _ 3.

	"type field values used during the mark phase of GC"
	HeaderTypeGC _ 2.
	GCTopMarker _ 3.  "neither an oop, nor an oop+1, this value signals that we have crawled back up to the top of the marking phase."

	"base header word bit fields"
	HashBits _ 16r1FFE0000.
	AllButHashBits _ 16rFFFFFFFF - HashBits.
	HashBitsOffset _ 17.
	SizeMask _ 16rFC.
	CompactClassMask _ 16r1F000.

	"masks for root and mark bits"
	MarkBit _ 16r80000000.
	RootBit _ 16r40000000.
	AllButMarkBit _ 16rFFFFFFFF - MarkBit.
	AllButRootBit _ 16rFFFFFFFF - RootBit.

	AllButMarkBitAndTypeMask _ AllButTypeMask - MarkBit.! !

Object subclass: #ObjectMemory
	instanceVariableNames: 'memory youngStart endOfMemory memoryLimit nilObj falseObj trueObj specialObjectsOop rootTable rootTableCount child field parentField freeBlock lastHash freeLargeContexts freeSmallContexts allocationCount lowSpaceThreshold signalLowSpace compStart compEnd fwdTableNext fwdTableLast remapBuffer remapBufferCount interruptCheckCounter checkAssertions allocationsBetweenGCs tenuringThreshold statFullGCs statFullGCMSecs statIncrGCs statIncrGCMSecs statTenures statRootTableOverflows displayBits '
	classVariableNames: 'AllButHashBits AllButMarkBit AllButMarkBitAndTypeMask AllButRootBit AllButTypeMask BaseHeaderSize CharacterTable ClassArray ClassBitmap ClassBlockContext ClassByteArray ClassCharacter ClassCompiledMethod ClassFloat ClassInteger ClassLargePositiveInteger ClassMessage ClassMethodContext ClassPoint ClassProcess ClassPseudoContext ClassSemaphore ClassString ClassTranslatedMethod CompactClassMask CompactClasses ConstMinusOne ConstOne ConstTwo ConstZero Done ExternalObjectsArray FalseObject FloatProto GCTopMarker HashBits HashBitsOffset HeaderTypeClass HeaderTypeFree HeaderTypeGC HeaderTypeShort HeaderTypeSizeAndClass LargeContextSize MarkBit MinimumForwardTableBytes NilContext NilObject RemapBufferSize RootBit RootTableSize SchedulerAssociation SelectorCannotReturn SelectorDoesNotUnderstand SelectorMustBeBoolean SizeMask SmallBlockContext SmallContextSize SmallMethodContext SpecialSelectors StackStart StartField StartObj TheDisplay TheFinalizationSemaphore TheInputSemaphore TheInterruptSemaphore TheLowSpaceSemaphore TheTimerSemaphore TrueObject TypeMask Upward '
	poolDictionaries: ''
	category: 'Squeak-Interpreter'!

