'From Squeak3.2alpha of 2 October 2001 [latest update: #4479] on 12 November 2001 at 1:48:51 pm'!
"Change Set:		eToyCmdOrder-sw
Date:			12 November 2001
Author:			Scott Wallace

Provides a mechanism for applying an externally-imposed ordering of phrases in an etoy viewer.  Uses this mechanism, illustratively, to impose new orderings on #basic and #scripting categories, with more to come as dictated by the real users of this stuff."!


!Object methodsFor: 'viewer' stamp: 'sw 11/12/2001 13:47'!
methodInterfacesInPresentationOrderFrom: interfaceList forCategory: aCategory
	"Answer the interface list sorted in desired presentation order, using a static master-ordering list, q.v.  The category parameter allows an escape in case one wants to apply different order strategies in different categories, but for now a single master-priority-ordering is used -- see the comment in method EToyVocabulary.masterOrderingOfPhraseSymbols"

	| masterOrder interfaces firstIndex secondIndex |
	masterOrder _ Vocabulary eToyVocabulary masterOrderingOfPhraseSymbols.
	interfaces _ interfaceList asSortedCollection:
		[:a :b |
			firstIndex _ masterOrder indexOf: a elementSymbol.
			secondIndex _ masterOrder indexOf: b elementSymbol.
			firstIndex > 0
				ifTrue:
					[secondIndex = 0 or: [secondIndex > firstIndex]]
				ifFalse:
					[secondIndex ==  0 or: [b elementSymbol < a elementSymbol]]].
	^ interfaces asArray
! !

!Object methodsFor: 'viewer' stamp: 'sw 11/12/2001 12:32'!
tilePhrasesForCategory: aCategorySymbol inViewer: aViewer
	"Return a collection of phrases for the category.  If using classic tiles, only include phrases that have fewer than two arguments, because all that they can handle."

	| interfaces itsSelector |
	interfaces _ self methodInterfacesForCategory: aCategorySymbol inVocabulary: aViewer currentVocabulary limitClass: aViewer limitClass.
	interfaces _ self methodInterfacesInPresentationOrderFrom: interfaces forCategory: aCategorySymbol.
	Preferences universalTiles ifFalse:
		[interfaces _ interfaces select:
			[:int |
				itsSelector _ int selector.
				itsSelector numArgs < 2 or:
					"The lone two-arg loophole in classic tiles"
					[#(color:sees:) includes: itsSelector]]].
	^ interfaces collect:
		[:aMethodInterface |
			aMethodInterface wantsReadoutInViewer
				ifTrue:
					[aViewer phraseForVariableFrom: aMethodInterface]
				ifFalse:
					[aViewer phraseForCommandFrom: aMethodInterface]]! !


!EToyVocabulary methodsFor: 'method list' stamp: 'sw 11/12/2001 13:45'!
masterOrderingOfPhraseSymbols
	"Answer a dictatorially-imposed presentation list of phrase-symbols.  This governs the order in which suitable phrases are presented in etoy viewers using the etoy vocabulary.  For any given category, the default implementation is that any items that are in this list will occur first, in the order specified here; after that, all other items will come, in alphabetic order by formal selector."

	^ #(beep: forward: turn: getX getY  getHeading
		startScript: pauseScript: stopScript: startAll: pauseAll: stopAll: tellAllSiblings: doScript:)! !

