'From Squeak 2.3 beta of Nov 25, 1998 on 11 January 1999 at 9:54 pm'!
"Change Set:		FlexMorphs
Date:			11 January 1999
Author:			Dan Ingalls

Introduces FlexMorphs which can be spawned by a menu item on BookPageThumbnails.

Three notable features:
Leverages off SketchMorphs to cache the flexed form.
Agressively (after any display) uncaches the original form (not the flexed one).
Borders are supported with flexing, even though the original may have no border.
"!

SketchMorph subclass: #FlexMorph
	instanceVariableNames: 'originalMorph borderWidth borderColor '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Basic'!

!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:57'!
addCustomMenuItems: aCustomMenu hand: aHandMorph

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	aCustomMenu addLine.
	aCustomMenu add: 'make a flex morph' selector: #makeFlexMorphFor: argument: aHandMorph.
	flipOnClick
		ifTrue: [aCustomMenu add: 'disable bookmark action' action: #toggleBookmark]
		ifFalse: [aCustomMenu add: 'enable bookmark action' action: #toggleBookmark].
	(bookMorph isKindOf: BookMorph)
		ifTrue:
			[aCustomMenu add: 'set page sound' action: #setPageSound:.
			aCustomMenu add: 'set page visual' action: #setPageVisual:]
! !

!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:10'!
makeFlexMorphFor: aHand

	aHand grabMorph: (FlexMorph new originalMorph: page)! !


!FlexMorph reorganize!
('all' addCustomMenuItems:hand: borderColor: borderWidth: changeBorderColor: changeBorderWidth: drawOn: extent: form generateRotatedForm initialize layoutChanged loadOriginalForm originalMorph: releaseCachedState updateFromOriginal)
!


!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 21:40'!
addCustomMenuItems: aCustomMenu hand: aHandMorph

	"super addCustomMenuItems: aCustomMenu hand: aHandMorph."
	aCustomMenu addLine.
	aCustomMenu add: 'update from original' action: #updateFromOriginal.
	aCustomMenu addList: #(('border color...' changeBorderColor:)
						('border width...' changeBorderWidth:)).
	aCustomMenu addLine.
! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 21:43'!
borderColor: aColor
	borderColor _ aColor.
	self updateFromOriginal! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 21:43'!
borderWidth: width
	borderWidth _ width asPoint.
	self updateFromOriginal! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 21:06'!
changeBorderColor: evt
	| aHand |
	aHand _ evt ifNotNil: [evt hand] ifNil: [self primaryHand].
	aHand changeColorTarget: self selector: #borderColor:.
! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 21:48'!
changeBorderWidth: evt
	| handle origin aHand |
	aHand _ evt ifNil: [self primaryHand] ifNotNil: [evt hand].
	origin _ aHand gridPointRaw.
	handle _ HandleMorph new forEachPointDo:
		[:newPoint | handle removeAllMorphs.
		handle addMorph:
			(PolygonMorph vertices: (Array with: origin with: newPoint)
				color: Color black borderWidth: 1 borderColor: Color black).
		self borderWidth: (newPoint - origin) abs // 5].
	aHand attachMorph: handle.
	handle startStepping! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:54'!
drawOn: aCanvas

	originalForm _ nil.  "Aggressively uncache the originalForm"
	^ super drawOn: aCanvas! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:35'!
extent: newExtent

	self loadOriginalForm.  "make sure it's not nil"
	^ super extent: newExtent! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:35'!
form

	self loadOriginalForm.  "make sure it's not nil"
	^ super form! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:35'!
generateRotatedForm

	self loadOriginalForm.  "make sure it's not nil"
	^ super generateRotatedForm! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 21:37'!
initialize
	super initialize.
	borderWidth _ 2@2.
	borderColor _ Color black.! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:35'!
layoutChanged

	self loadOriginalForm.  "make sure it's not nil"
	^ super layoutChanged! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:34'!
loadOriginalForm

	originalForm ifNil: [self updateFromOriginal].
! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 20:19'!
originalMorph: aMorph

	originalMorph _ aMorph.
	scalePoint _ 0.25@0.25.
	self updateFromOriginal.! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 19:44'!
releaseCachedState
	"Clear cache of rotated, scaled Form."

	originalForm _ Form extent: 10@10.  "So super hibernate won't have to work hard
												but won't crash either."
	super releaseCachedState.
	rotatedForm _ nil.
	originalForm _ nil.! !

!FlexMorph methodsFor: 'all' stamp: 'di 1/11/1999 21:46'!
updateFromOriginal

	| intermediateForm |
	intermediateForm _ originalMorph imageForm offset: 0@0.
	intermediateForm border: intermediateForm boundingBox
		widthRectangle: (borderWidth corner: borderWidth+1)
		rule: Form over fillColor: borderColor.
	self form: intermediateForm.
	originalMorph fullReleaseCachedState! !


FlexMorph removeSelector: #originalForm!
