'From Squeak 2.3 of January 14, 1999 on 8 March 1999 at 10:42:37 pm'!
"Change Set:		updatingButton-sw
Date:			8 March 1999
Author:			Scott Wallace

Adds a variant of the three-phase-button morph for expressing and controlling boolean values which may also be changed externally.  

Puts an empty #mouseUpBalk: method in class Object to enable such a button to be used with any kind of object as its target, and removes the now-redundant empty #mouseUpBalk: methods lying around here and there in the system.

Adds a new kind of control allowing regulation of, and reporting of, a single Preference (used in the scripting template) -- this format might eventually be nicer to have in the generic preferences control panel.  This control works fine even when  the preference has its value changed somewhere else"!

ThreePhaseButtonMorph subclass: #UpdatingThreePhaseButtonMorph
	instanceVariableNames: 'getSelector '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Widgets'!

!Object methodsFor: 'user interface' stamp: 'sw 3/8/1999 15:27'!
mouseUpBalk: evt
	"A button I own got a mouseDown, but the user moved out before letting up.  Certain kinds of objects (so-called 'radio buttons', for example, and other structures that must always have some selection, e.g. PaintBoxMorph) wish to take special action in this case; this default does nothing."
! !


!Preferences class methodsFor: 'initialization' stamp: 'sw 3/8/1999 14:20'!
chooseInitialSettings
	"Restore the default choices for Preferences."
	"Preferences chooseInitialSettings"
	#(	(allowSoundQuickStart				false)
		(allowSysWindowEmbedding			false)
		(automaticViewerPlacement			true)
		(balloonHelpEnabled					true)
		(browseWithPrettyPrint				false)
		(cmdDotEnabled						true)
		(confirmFirstUseOfStyle				true)
		(disableSounds						false)
		(fastDragWindowForMorphic			false)
		(fenceEnabled						true)
		(ignoreStyleIfOnlyBold				true)
		(inboardScrollbars					false)
		(logDebuggerStackToFile				true)
		(mouseOverHaloseEnabled			false)
		(noviceMode							false)
		(printAlternateSyntax				false)
		(reverseWindowStagger				true)
		(showDebugHaloHandle				true)
		(showDiffsInChangeList				true)
		(showTimeStampsInMenuTitles		false)
		(showProjectZoom					false)
		(suppressCheckForSlips				false)
		(suppressUpdateServerPrompt		false)
		(thoroughSenders					true)
		(unlimitedPaintArea					false)
		(updateSavesFile						false)
		(useAnnotationPanes				false)
		(useGlobalFlaps						true)
		(warnIfNoChangesFile				true)
		(warnIfNoSourcesFile				true))
	do:
		[:aPair |
			aPair last == #true
				ifTrue:
					[self enable: aPair first]
				ifFalse:
					[self disable: aPair first]]

! !

!Preferences class methodsFor: 'pref buttons' stamp: 'sw 3/8/1999 15:20'!
buttonRepresenting: prefSymbol wording: aString color: aColor
	"self currentHand attachMorph: (Preferences buttonRepresenting: #balloonHelpEnabled wording: 'Balloon Help' color: nil)"
	"Return a button that controls the setting of prefSymbol.  It will keep up to date even if the preference value is changed in a different place"
	| outerButton aButton aForm |
	(FlagDictionary includesKey: prefSymbol) ifFalse: [self error: 'Unknown preference: ', prefSymbol printString].
	outerButton _ AlignmentMorph newRow height: 24.
	outerButton color:  (aColor ifNil: [Color r: 0.645 g: 1.0 b: 1.0]).
	outerButton hResizing: #spaceFill; vResizing: #shrinkToFit.
	outerButton addMorph: (aButton _ UpdatingThreePhaseButtonMorph new).
	aButton onImage: (aForm _ ScriptingSystem formAtKey: #ButtonOn);
		pressedImage:	aForm;
		offImage: (ScriptingSystem formAtKey: #ButtonOff);
		actionSelector: #togglePreference:;
		extent: 20 @ 20;
		arguments: (Array with: prefSymbol);
		target: Preferences;
		getSelector: prefSymbol.
	outerButton addTransparentSpacerOfSize: (2 @ 0).
	outerButton addMorphBack: (StringMorph contents: aString font: ScriptingSystem fontForScriptorButtons) lock.
	outerButton setBalloonText: (Preferences helpMessageForPreference: prefSymbol).

	^ outerButton
	

! !

!Preferences class methodsFor: 'pref buttons' stamp: 'sw 3/8/1999 14:03'!
togglePreference: prefSymbol
	| curr |
	curr _ (FlagDictionary at: prefSymbol ifAbsent: [^ self error: 'unknown pref: ', prefSymbol printString]).
	FlagDictionary at: prefSymbol put: (curr == true) not

! !


!ThreePhaseButtonMorph methodsFor: 'all' stamp: 'sw 3/8/1999 13:56'!
isOn
	^ state == #on! !

Smalltalk renameClassNamed: #UpdatedingThreePhaseButtonMorph as: #UpdatingThreePhaseButtonMorph!

!UpdatingThreePhaseButtonMorph reorganize!
('all' getSelector: step wantsSteps)
!


!UpdatingThreePhaseButtonMorph methodsFor: 'all' stamp: 'sw 3/8/1999 13:50'!
getSelector: sel
	getSelector _ sel! !

!UpdatingThreePhaseButtonMorph methodsFor: 'all' stamp: 'sw 3/8/1999 13:58'!
step
	| newBoolean |
	state == #pressed ifTrue: [^ self].
	newBoolean _ target perform: getSelector.
	newBoolean == self isOn
		ifFalse:
			[self state: (newBoolean ifTrue: [#on] ifFalse: [#off])]! !

!UpdatingThreePhaseButtonMorph methodsFor: 'all' stamp: 'sw 3/8/1999 13:50'!
wantsSteps
	^ true! !


Morph removeSelector: #mouseUpBalk:!
Presenter removeSelector: #mouseUpBalk:!
StandardScriptingSystem removeSelector: #mouseUpBalk:!
