'From Squeak 2.3 of January 14, 1999 on 7 April 1999 at 9:06:06 pm'!
"Change Set:		scriptingSpace
Date:			6 April 1999
Author:			Scott Wallace

A few adjustments to the standard scripting space:

Removes the special-purpose scraps tab from future prebuilt-scripting-areas and generally from tabbed palettes. though it may persist residually in already-existing structures.

Adds a 'tiles' tab to the palette, thus making some tiles that are not directly available in the viewer more accessible.

Fixes the long-standing bug whereby some clicks on tabs would add the related morph beneath, rather than instead of, a viewer.

CAUTION: in order for the new scripting spaces to be accessible, their prototypes in the standard parts bin and the default Tools flap need to be up to date, and this is done by reinitializing those two structures.  In the latter case, this could clobber existing flap artifacts in the user's image which might have been customized -- sorry!! "!


!StandardScriptingSystem methodsFor: 'utilities' stamp: 'sw 4/7/1999 11:58'!
newScriptingSpace
	"Answer a complete scripting space"

	| aTemplate  palette aPlayfield aWindow |
	aWindow _ (SystemWindow labelled: 'scripting area') model: ScriptingDomain new.
	aWindow extent: 640 @ 480.
	aTemplate _ PasteUpMorph new setNameTo: 'etoy'.
	aTemplate extent: 638 @ 470.
	aWindow addMorph: aTemplate frame: (0@0 corner: 1@1).
	aTemplate setStandardTexture.
	aTemplate impartPrivatePresenter; setProperty: #automaticPhraseExpansion toValue: true; beSticky.
	aTemplate presenter addStopStepGoButtons; addTrashCan.

	palette _ TabbedPalette new pageSize: 200@320.
	palette beSticky.
	aTemplate addMorph: palette.
	palette addMenuTab.
	palette addPartsTab.
	palette addControlsTab.
	palette addTilesTab.
	palette becomeStandardPalette.
	aPlayfield _ PasteUpMorph new setNameTo: 'playfield'.
	(Preferences valueOfFlag: #eToyScheme)  "Not widely advertised"
		ifFalse:	
			[aPlayfield setToAdhereToEdge: #topRight.
			palette setToAdhereToEdge: #topLeft]
		ifTrue:
			[aPlayfield setToAdhereToEdge: #topLeft.
			palette setToAdhereToEdge: #topRight].

	aTemplate addMorph: aPlayfield.
	aPlayfield extent: 340@300; position: aTemplate topRight - (400@0).
	aPlayfield beSticky.
	aPlayfield automaticViewing: true.
	aPlayfield wantsMouseOverHalos: true.
	aTemplate presenter standardPlayfield: aPlayfield.
	
	^ aWindow

! !


!TabbedPalette methodsFor: 'parts & controls tabs' stamp: 'sw 4/7/1999 11:59'!
addTilesTab
	| aPage |
	aPage _ self presenter tilesPagesForPartsBin first.
	aPage extent: pageSize.
	aPage setNameTo: 'tiles'.
	self addTabForBook: aPage withBalloonText: 'useful tiles for scripting'
! !

!TabbedPalette methodsFor: 'palette menu' stamp: 'sw 3/24/1999 10:34'!
addMenuTab
	| aMenu aTab aGraphic sk |
	aMenu _ MenuMorph new defaultTarget: self.
	aMenu stayUp: true.
	aMenu add:  'clear' action: #showNoPalette.
	aMenu add:  'sort tabs' action: #sortTabs:.
	aMenu add:  'choose new colors for tabs' action: #recolorTabs.
	aMenu setProperty: #paletteMenu toValue: true.
	aMenu add:  'make me the Standard palette' action: #becomeStandardPalette.
	aTab _ self addTabForBook: aMenu  withBalloonText: 'a menu of palette-related controls'.
	aTab highlightColor: tabsMorph highlightColor; regularColor: tabsMorph regularColor.
	tabsMorph laySubpartsOutInOneRow; layoutChanged.

	aGraphic _ ScriptingSystem formAtKey: 'Menu'.
	aGraphic ifNotNil:
		[aTab removeAllMorphs.
		aTab addMorph: (sk _ SketchMorph new form: aGraphic).
		sk position: aTab position.
		sk lock.
		aTab fitContents].
	self layoutChanged! !

!TabbedPalette methodsFor: 'palette menu' stamp: 'sw 3/24/1999 10:37'!
showPaletteMenu
	| aMenu evt |
	evt _ self primaryHand lastEvent.
	aMenu _ MenuMorph new defaultTarget: self.

	aMenu add:  'clear' action: #showNoPalette.
	aMenu addLine
.
	aMenu add:  'sort tabs' action: #sortTabs:.
	aMenu add:  'choose new colors for tabs' action: #recolorTabs.
	(self presenter standardPaletteIsAlready: self) ifFalse:
		[aMenu add:  'make me the Standard palette' action: #becomeStandardPalette].

	aMenu addTitle: 'Palette'.
	aMenu popUpAt: self cursorPoint event: evt! !

!TabbedPalette methodsFor: 'miscellaneous' stamp: 'sw 4/7/1999 12:16'!
replaceSubmorph: oldMorph by: newMorph
	super replaceSubmorph: oldMorph by: newMorph.
	oldMorph == currentPage ifTrue:
		[currentPage _ newMorph]! !


!TrashCanMorph methodsFor: 'event handling' stamp: 'sw 4/7/1999 14:47'!
mouseDown: evt
	| palette |
	self currentHand endDisplaySuppression.
	palette _ self standardPalette.
	((palette notNil and: [palette isInWorld]) and: [palette hasScrapsTab])
		ifTrue:
			[palette showScrapsTab]
		ifFalse:
			[self currentHand openScrapsBook]
! !

!TrashCanMorph methodsFor: 'dropping' stamp: 'sw 3/24/1999 10:37'!
acceptDroppingMorph: aMorph event: evt

	Preferences soundsEnabled ifTrue: [self class playDeleteSound].
	evt hand endDisplaySuppression.
	self state: #off.
	aMorph delete.
	aMorph == Utilities scrapsBook ifFalse:
		[Utilities addToTrash: aMorph]! !


!TrashCanMorph class methodsFor: 'miscellaneous' stamp: 'sw 3/24/1999 10:36'!
moveToTrash: aMorph
	Preferences soundsEnabled ifTrue: [self playDeleteSound].
	aMorph delete.
	aMorph == Utilities scrapsBook ifFalse:
		[Utilities addToTrash: aMorph]! !


TabbedPalette removeSelector: #addViewerTab!
TabbedPalette removeSelector: #addScrapsTab!
TabbedPalette removeSelector: #addToTrash:!
"Postscript:
"
ScriptingSystem resetStandardPartsBin.
Utilities reinstateDefaultFlaps.!

