'From Squeak3.9alpha of ''2 November 2004'' [latest update: #6536] on 30 December 2004 at 12:35:27 am'!
"Change Set:		UnstableUpdtsSrvr-dew
Date:			27 December 2004
Author:			Doug Way

Create a new UnstableUpdatesAtSqF ServerDirectory (in the postscript), to allow broadcasting updates directly to the unstable stream.  This is the same machine as the (stable) Squeak Public Updates server.  Only the groupName needs to be different, in order to update the UnstableUpdates.list file instead of Updates.list.

Also fix an error-reporting bug in #checkServersWithPrefix:andParseListInto:.

Also remove the UIUC url from the unstable Utilities updatesUrl list, since it hasn't worked in years.  This speeds up updating considerably since we don't have to wait for that server to timeout."
!


!ServerDirectory methodsFor: 'updates' stamp: 'dew 12/27/2004 23:51'!
checkServersWithPrefix: prefix andParseListInto: listBlock
	"Check that all servers are up and have the latest Updates.list.
	Warn user when can't write to a server that can still be read.
	The contents of updates.list is parsed into {{vers. {fileNames*}}*},
	and returned via the listBlock."

	|  serverList updateLists listContents maxSize outOfDateServers |
	serverList _ self serversInGroup.
	serverList isEmpty
		ifTrue: [^Array new].

	updateLists := Dictionary new.
	serverList do: [:updateServer |
		[listContents := updateServer getFileNamed: prefix , 'updates.list'.
		updateLists at: updateServer put: listContents]
			on: Error
			do: [:ex |
				(PopUpMenu labels: 'Cancel entire update' withCRs)
					startUpWithCaption: 'Server ', updateServer moniker,
					' is unavailable.\Please consider phoning the administator.\' withCRs, ex printString.
				^Array new]].

	maxSize := (updateLists collect: [:each | each size]) max.
	outOfDateServers := updateLists keys select: [:updateServer |
		(updateLists at: updateServer) size < maxSize].

	outOfDateServers do: [:updateServer |
		(self outOfDate: updateServer) ifTrue: [^Array new]].

	listBlock value: (Utilities parseListContents: listContents).

	serverList removeAll: outOfDateServers.
	^serverList

! !

"Postscript:"
| unstableUpdatesServer unstableUrls |

"Add new unstable updates server."
unstableUpdatesServer _ (ServerDirectory serverNamed: 'UpdatesAtSqF') copy.
unstableUpdatesServer groupName: 'Unstable Squeak Updates*'.
ServerDirectory addServer: unstableUpdatesServer named: 'UnstableUpdatesAtSqF'.

"Remove UIUC as an update url from the unstable url list (it hasn't worked in years, and you have to wait for it to timeout before it can proceed to the next server).  Also rename the url group as 'Unstable Squeak Updates*' to match the server name (a minor thing)."
unstableUrls _ Utilities updateUrlLists detect: [:urlColl | urlColl first = 'Unstable Updates*'] ifNone: [nil].
unstableUrls ifNotNil:
	[unstableUrls at: 1 put: 'Unstable Squeak Updates*'.
	unstableUrls at: 2 put: #('update.squeakfoundation.org/external/')].
!

