'From Squeak2.9alpha of 12 June 2000 [latest update: #3051] on 30 November 2000 at 7:46:21 pm'!
"Change Set:		BenchmarksAgain
Date:			30 November 2000
Author:			Dan Ingalls

Mainly a clarification.  Adds the following comment to SystemDictionary>>macroBenchmarks:

PLEASE TAKE NOTE: The goal of these benchmarks is to provide a simple basis for A/B performance comparisons with a given Squeak image.  For example JIT vs interpreter, new GC vs old, etc.  However, a number of these benchmarks will drift with evolution of the Squeak image, as, for instance, if the number of methods decompiled in macroBenchmark1 were to change.  Therefore it is essential NEVER to make comarisons between macroBenchmarks run from two different images.
"!


!SystemDictionary methodsFor: 'housekeeping' stamp: 'di 11/30/2000 19:43'!
macroBenchmarks 
	"Reports an array of times taken to run a number of macro operations indicative of typical Squeak activity."

	"PLEASE TAKE NOTE: The goal of these benchmarks is to provide a simple basis for A/B performance comparisons with a given Squeak image.  For example JIT vs interpreter, new GC vs old, etc.  However, a number of these benchmarks will 'drift' with evolution of the Squeak image, as, for instance, if the number of methods decompiled in macroBenchmark1 were to change.  Therefore it is essential *never* to make comarisons between macroBenchmarks run from two different images."

	"Smalltalk macroBenchmarks #(43215 53122 81336 26927 8993 12607 9024) 400MHz G3"

	| interp time saveMorphs freeCell report |
	^ {
	"1:	Decompile, pretty-print, and compile a bunch of methods.
		Does not install in classes, so does not flush cache."
	Time millisecondsToRun: [Smalltalk macroBenchmark1].

	"2:	Build morphic tiles for all methods over 800 bytes (;-).
		Does no display."
	Time millisecondsToRun: [SyntaxMorph testAllMethodsOver: 800].

	"3:	Translate the interpreter with inlining.
		Does not include any plugins."
	Time millisecondsToRun: [Smalltalk macroBenchmark2].

	"4:	Run the context step simulator.
		200 iterations printing pi and 15 factorial."
	Time millisecondsToRun: [Smalltalk macroBenchmark3].

	"5:	Run the InterpreterSimulator for 150,000 bytecodes.
		Will only run if you have mini.image in your directory."
	(FileDirectory default includesKey: 'mini.image')
		ifTrue: [interp _ InterpreterSimulator new openOn: 'mini.image'.
				time _ Time millisecondsToRun: [interp runForNBytes: 150000].
				interp close. Display restore.
				time]
		ifFalse: [0].

	"6:	Open 10 browsers and close them.
		Includes browsing to a specific method."
	Smalltalk isMorphic
		ifTrue: [saveMorphs _ self currentWorld submorphs.
				self currentWorld removeAllMorphs.  "heh, heh"
				time _ Time millisecondsToRun:
					[1 to: 10 do: [:i | Browser fullOnClass: SystemDictionary selector: #macroBenchmarks].
					self currentWorld submorphs do:
						[:m | m delete. self currentWorld doOneCycle]].
				self currentWorld addAllMorphs: saveMorphs.
				time]
		ifFalse: [0].

	"7:	Play a game of FreeCell with display, while running the MessageTally.
		Thanks to Bob Arning for the clever part of this one."
	Smalltalk isMorphic
		ifTrue: ["Play a trivial game of FreeCell with MessageTally and report."
				(freeCell _ FreeCell new) openInWorld.
				time _ Time millisecondsToRun: [freeCell board pickGame: 1].
				(((report _ self currentWorld firstSubmorph) isKindOf: SystemWindow)
					and: [self currentWorld firstSubmorph label = 'Spy Results'])
					ifTrue: [report delete].
				freeCell delete.
				time]
		ifFalse: [0].
		}
"
If you have the right image then run this and print the tinyBenchmarks in the simulation window.  Note that you have to use the menu printIt rather than cmd-p...
	(InterpreterSimulator new openOn: 'mini.image') test

While you are at it, print out the result of
	ContextPart runSimulated: [0 tinyBenchmarks]
"! !

